classdef CCDemo < handle
    %PADDLEAPP Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        gui = [];
        icub = [];
        icubUI = [];
        stateUI = [];
        cntUI = [];
        tactileAssistant = [];
        %markerAssistant = [];
        %gazingAssistant = [];
        icubphrases;
        
        numMarkers = 9;
        skin = [];
        net = [];
        nnSamplingTime = 0.25;
        nnSimTime = 3600;
        
        data = {};
        
        status = 0;
        PRECONNECT = -1;
        WAITFORSTART = 0;
        CONNECTING = 1;
        CALIBTACTILES = 2;
        RANDHEADMOV = 3;
        SIMPLEGAZE = 4;
        PERFORMACTION = 5;
        RUNNING = 6;
        
        cntparams = {
            'MarkerTracker', '/CCDemo/MarkerTracker:i', '/MarkerTracker/marker:o', 'input';...
            'Tactiles', '/CCDemo/Tactiles:i', '/icub/skin/right_arm', 'input';...
            'RandomHeadMovements', '/CCDemo/RandomHeadMovements/state:o', '/RandomHeadMovements/state:i', 'output';...
            'SimpleGazeCtrl', '/CCDemo/SimpleGazeCtrl/state:o', '/SimpleGazeCtrl/command', 'output';
            };
    end
    
    methods
        function app = CCDemo()
            app.gui = CCDemoGUI(app);
            app.icubUI = ICubUISimRob(app, app.gui.icubPanel);
            app.cntUI = YarpCntAssistant(app.cntparams, app.gui.varPanel);
            app.cntUI.hideUI();
            app.stateUI = StateMachineUI(app, app.gui.statePanel);
            %app.icub = ICub(app.icubUI.icubnames{app.icubUI.SIMIDX}, app.icubUI.bodyparts);
            app.tactileAssistant = TactileAssistant(app, app.gui.varPanel);
            
            app.stateUI.setState(app.PRECONNECT);
            
            app.skin = ICubSkin();
            app.icubphrases = load('icub-phrases');
            app.net = neunet(6, app.numMarkers, 1, app.nnSamplingTime, app.nnSimTime, 1);
        end
        
        function running(app,src,evn)
            if get(src, 'Value')
                %app.stateUI.setState(app.RUNNING);
                disp('running...');
                
                app.net.createWindow();
                
                yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
                yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
                yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set start random');
                
                tic;
                steps = 0;
                postaction = 0;
                numStepsForAction = 70;
                while get(src, 'Value')
                    %get markers
                    marker = zeros(1,app.numMarkers);
                    [mbottle mstr] = yarpRead(app.cntUI.cnts.MarkerTracker.port);
                    numDetectedMarker = str2num(mbottle.get(0).toString_c);
                    for i=1:numDetectedMarker
                        dmidx = str2num(mbottle.get(3*(i-1)+1).toString_c);
                        marker(dmidx) = 1;
                    end
                    
                    %read tactiles
                    [mbottle mstr] = yarpRead(app.cntUI.cnts.Tactiles.port);
                    touched = app.skin.touched(str2num(mstr));
                    
                    %update network
                    app.net.update(marker', touched);
                    actions = app.net.smile;
                    trStimulus = app.net.stimulusToTrack;
                    if actions
                        disp('network triggered action!');
                        %currentStep = app.net.currentStep
                        %tmp = app.net.log.stimuliProcessed(app.net.currentStep - 2, :)
                    end
                    if trStimulus >= 0
                        disp(['stimulus to track ' num2str(trStimulus)]);
                    end
                    if mod(steps, 10) == 1
                        app.net.refreshGraphs();
                    end
                    %actions = 0;
                    %trStimulus = 0;
                    
                    %execute actions
                    %smile
                    if (touched || actions(1)) && ~postaction
                        postaction = steps;
                        app.icubUI.setEmotion('hap');
                    end
                    %gaze to person (preprogrammed)
                    if touched && ~actions(1)
                        phrase = randi(5,1);
                        sound(app.icubphrases.phrasesUS(phrase,'44100'));
                        %TODO call RandomHeadMovement for VITE motion to desired head configuration
                        %yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, ['1 ' num2str([pan tilt])]);
                        app.performaction();
                    end
                    %track person
                    if actions(1)
                        phrase = randi(2,1);
                        sound(app.icubphrases.phrasesCS(phrase,'44100'));
                        yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set stop');
                        yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, ['set markerToTrack ' num2str(trStimulus - 1)]);
                        yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 1');
                        %TODO stop tracking after a few seconds...
                    end
                    
                    %utility
                    steps = steps + 1;
                    if (steps-postaction > numStepsForAction) && postaction
                        postaction = 0;
                        %reset facial expression
                        app.icubUI.setEmotion('clr');
                        %stop simplegazectrl
                        yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
                        yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
                        %start random head movements
                        yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set start random');
                    end
                    pause(0.001);
                end
                et = toc;
                disp(['iteration with ' num2str(steps/et) ' Hz']);
                fprintf('done!\n');
            else
                app.stateUI.setState(app.WAITFORSTART);
                yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set stop');
                yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
                app.icubUI.setEmotion('clr');
                close(app.net.graph.f1);
            end
        end
        
        function connect(app,src,evn)
            if get(src, 'Value')
                app.stateUI.setState(app.CONNECTING);
            else
                %if ~app.cntUI.isconnected() && ~app.icubUI.isconnected()
                %    app.stateUI.setState(app.PRECONNECT);
                %else
                app.stateUI.setState(app.WAITFORSTART);
                %end
            end
        end
        
        function calibtactiles(app,src,evn)
            if get(src, 'Value')
                app.stateUI.setState(app.CALIBTACTILES);
            else
                app.stateUI.setState(app.WAITFORSTART);
            end
        end
        
        function randheadmov(app,src,evn)
            if get(src, 'Value')
                app.stateUI.setState(app.RANDHEADMOV);
                yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set start random');
                yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set alpha 5');
            else
                yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set stop');
                app.stateUI.setState(app.WAITFORSTART);
            end
        end
        
        function simplegazectrl(app,src,evn)
            if get(src, 'Value')
                app.stateUI.setState(app.SIMPLEGAZE);
                %TODO get marker ID from simplegazeAssistant
                
                %% TEST
                trStimulus =5;
                yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, ['set markerToTrack ' num2str(trStimulus)]);
                yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 1');
            else
                yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
                yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
                yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
                app.stateUI.setState(app.WAITFORSTART);
            end
        end
        
        function performaction(app,src,evn)
            yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
            yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
            
            %smile
            app.icubUI.setEmotion('hap');
            
            %gaze to person (preprogrammed)
            %TODO call RandomHeadMovement for VITE motion to desired head configuration
            %yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, ['1 ' num2str([pan tilt])]);
            
            q = [15 0 0 0 0 0];
            yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set alpha 10');
            yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, ['set target ' num2str(deg2rad(q))]);
            %yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set stop');
            %app.icubUI.driveToPosture('head', deg2rad(q));
            
            if 0
            idx =find(cell2mat(app.icubUI.icubexe)==1, 1,'last');
            icub = app.icubUI.icub{idx};
            if icub.connected
                q = rad2deg(icub.getPosture('head')); % 20 deg look up
                q(1) =10;
                yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, ['set target ' num2str(deg2rad(q))]);
            end
            end
        end
        
        function delete(app)
            delete(app.icubUI);
            delete(app.cntUI);
            %if ~isempty(app.icub)
            %    delete(app.icub);
            %end
        end
    end
    
end

