classdef CCDemoSim < handle
    %PADDLEAPP Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        gui;
        icub;
        icubUI;
        stateUI;
        cntUI;
        tactileAssistant;
        %markerAssistant = [];
        %gazingAssistant = [];
        skin;
        
        netCC;
        netOC;
        
        nrCSStimuli         = 9;                    % also the number of markers
        nrOCActions         = 9;                    % also the number of output for the OC net
        nrOCCases           = 2;                    % also the number of commands
        OCStimulus          = 0;                    % which OC command is being currently seen
        OCStimuli           = zeros(1,2);          % vector of OC commends/stimuli
        nnSamplingTime      = 0.25;
        smileDuration       = 2.5;                  % in seconds
        absenceDetectedAfter = 5;                   % after this time without CS stimulus, the mode goes back from OC to CC
        learningMode        = 1;                    % 1 for CC and 2 for OC
        nnSimTime           = 3600;
        randomSeed          = 1;
        trackingStimulus    = -1;
        waitingForNextAction = 0;
        touched = 0;

        data = {};
        
        status          = 0;
        PRECONNECT      = -1;
        WAITFORSTART    = 0;
        CONNECTING      = 1;
        CALIBTACTILES   = 2;
        RANDHEADMOV     = 3;
        SIMPLEGAZE      = 4;
        PERFORMACTION   = 5;
        RUNNING         = 6;
        icubphrases;
        cntparams = {
            'MarkerTracker', '/CCDemo/MarkerTracker:i', '/MarkerTracker/marker:o', 'input';...
            'RandomHeadMovements', '/CCDemo/RandomHeadMovements/state:o', '/RandomHeadMovements/state:i', 'output';...
            'SimpleGazeCtrl', '/CCDemo/SimpleGazeCtrl/state:o', '/SimpleGazeCtrl/command', 'output';
            };
    end
    
    methods
        function app = CCDemoSim()
            app.gui = CCDemoGUI(app);
            app.icubUI = ICubUISimRob(app, app.gui.icubPanel);
            app.cntUI = YarpCntAssistant(app.cntparams, app.gui.varPanel);
            app.cntUI.hideUI();
            app.stateUI = StateMachineUI(app, app.gui.statePanel);
            %app.icub = ICub(app.icubUI.icubnames{app.icubUI.SIMIDX}, app.icubUI.bodyparts);
            app.tactileAssistant = TactileAssistant(app, app.gui.varPanel);
            
            app.stateUI.setState(app.PRECONNECT);
            app.icubphrases = load('icub-phrases');
            
            set(app.gui.f, 'KeyPressFcn', @app.keypressed);
            
            %app.skin = ICubSkin();
            app.netCC = neunet(app.randomSeed, app.nrCSStimuli, 1, app.nnSamplingTime, app.nnSimTime, 1);
            app.netOC = neunet(app.randomSeed, app.nrOCCases, app.nrOCActions, app.nnSamplingTime, app.nnSimTime, 2);
        end
        
        function running(app,src,evn)
            if get(src, 'Value')
                %app.stateUI.setState(app.RUNNING);
                disp('running...');
                
                app.netCC.createWindow();
                
                yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
                yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
                yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set start random');
                
                tic;
                steps               = 0;
                smileExpiresIn      = -1;
                
                while get(src, 'Value')
                    
                    %read tactiles (simulated by key listener)
                    if app.touched %written by key listener
                        touchReceived = 1;
                        disp('received touched');
                        app.touched = 0; %reset
                    else
                        touchReceived = 0;
                    end
                    
                    %read stimuli (i.e. markers) used both in CC and OC scenarios
                    marker = zeros(1,app.nrCSStimuli);
                    [mbottle mstr] = yarpRead(app.cntUI.cnts.MarkerTracker.port);
                    numDetectedMarker = str2num(mbottle.get(0).toString_c);
                    for i=1:numDetectedMarker
                        dmidx = str2num(mbottle.get(3*(i-1)+1).toString_c)+1;
                       % disp(['Markers detected: ' num2str(dmidx)]);
                        marker(dmidx) = 1;
                    end          
                    
                    if touchReceived
                        phrase = randi(5,1);
                        sound(app.icubphrases.phrasesUS(phrase,:),44100);
                        
                        app.icubUI.setEmotion('hap');
                        smileExpiresIn = round(app.smileDuration / app.samplingTime);
                    end
                    
                    
                    if app.learningMode == 1
                        % CLASSICAL CONDITIONING MODE, DEFAULT MODE WHEN NO CS IS PRESENT
                        
                        % update network
                        app.netCC.update(marker', touchReceived);
                                                  
                        % CS triggered
                        if app.netCC.CS
                            app.icubUI.setEmotion('hap');
                            smileExpiresIn = round(app.smileDuration / app.samplingTime);

                            phrase = randi(2,1);
                            sound(app.icubphrases.phrasesCS(phrase,:),44100);
                            
                        end
                        
                        app.trackingStimulus = app.netCC.stimulusToTrack;
                      
                        if app.netCC.CS && app.trackingStimulus >= 0
                            
                            disp(['Starting to track stimulus ' num2str(app.trackingStimulus)]);
 
                            yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set stop');
                            yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, ['set markerToTrack ' num2str(app.trackingStimulus - 1)]);
                            yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 1');
                            
                            % enter OC mode
                            app.learningMode = 2;
                        end
                    else
                        % OPERANT CONDITIONING MODE, TRIGGERED BY CS
                        
                        % reading stimuli
                        app.OCStimuli = [0 0];
                        if app.OCStimulus > 0
                            if app.OCStimulus == 1
                                app.OCStimuli = [1 0];
                            end
                            if app.OCStimulus == 2
                                app.OCStimuli = [0 1];
                            end
                            app.OCStimulus = 0;
                        end
                        
                        if marker(app.trackingStimulus) == 0
                            CSabsentFor = CSabsentFor + 1;
                        else
                            CSabsentFor = 0;
                        end
                        
                        % update network
                        app.netOC.update(app.OCStimuli, touchReceived);
                        
                        % play action
                        sayColour = find(app.netOC.action == 1, 1);
                        if ~isempty(sayColour) && app.waitingForNextAction <= 0
                            sound(app.icubphrases.phrasesCS(phrase,:),44100);
                            app.waitingForNextAction = app.actionTime / app.samplintTime;
                        end
                        
                        % CS is absent, return to CC learning mode
                        if CSabsentFor > app.CSabsenceDetectedAfter / app.samplingTime
                            app.learningMode = 1;
                            app.trackingStimulus = -1;
                            CSabsentFor = 0;
                            
                            yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
                            %start random head movements
                            yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set start random');
                            
                        end
                    end
                    
                    if smileExpiresIn == 0
                        %reset facial expression
                        app.icubUI.setEmotion('clr');
                        %stop simplegazectrl
                        yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
                        yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
                        %start random head movements
                        yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set start random');
                    end
                    
                    steps = steps + 1;
                    smileExpiresIn = smileExpiresIn - 1;
                    app.waitingForNextAction = app.waitingForNextAction - 1;
                    
                    if mod(steps, 20) == 1
                        app.netCC.refreshGraphs();
                    end
                    if mod(steps, 20) == 10
                        app.netOC.refreshGraphs();
                    end
                    
                    pause(0.01); %if keylistener not working, increase pause!
                end
                et = toc;
                disp(['iteration with ' num2str(steps/et) ' Hz']);
                fprintf('done!\n');
            else
                app.stateUI.setState(app.WAITFORSTART);
                yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set stop');
                yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
                app.icubUI.setEmotion('clr');
                close(app.netCC.graph.f1);
            end
        end
        
        function connect(app,src,evn)
            if get(src, 'Value')
                app.stateUI.setState(app.CONNECTING);
            else
                %if ~app.cntUI.isconnected() && ~app.icubUI.isconnected()
                %    app.stateUI.setState(app.PRECONNECT);
                %else
                app.stateUI.setState(app.WAITFORSTART);
                %end
            end
        end
        
        function calibtactiles(app,src,evn)
            app.stateUI.setState(app.WAITFORSTART);
        end
        
        function randheadmov(app,src,evn)
            if get(src, 'Value')
                app.stateUI.setState(app.RANDHEADMOV);
                yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set start random');
                yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set alpha 5');
            else
                yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set stop');
                app.stateUI.setState(app.WAITFORSTART);
            end
        end
        
        function simplegazectrl(app,src,evn)
            if get(src, 'Value')
                app.stateUI.setState(app.SIMPLEGAZE);
                %TODO get marker ID from simplegazeAssistant
                
                %% TEST
                trStimulus =2;
                yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, ['set markerToTrack ' num2str(trStimulus)]);
                yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 1');
            else
                yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
                yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
                yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
                app.stateUI.setState(app.WAITFORSTART);
            end
        end
        
        function performaction(app,src,evn)
            yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
            yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
            
            %smile
            app.icubUI.setEmotion('hap');
            
            %gaze to person (preprogrammed)
            %TODO call RandomHeadMovement for VITE motion to desired head configuration
            %yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, ['1 ' num2str([pan tilt])]);
            
            q = [15 0 0 0 0 0];
            yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set alpha 10');
            yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, ['set target ' num2str(deg2rad(q))]);
            %yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set stop');
            %app.icubUI.driveToPosture('head', deg2rad(q));
            
            if 0
                idx =find(cell2mat(app.icubUI.icubexe)==1, 1,'last');
                icub = app.icubUI.icub{idx};
                if icub.connected
                    q = rad2deg(icub.getPosture('head')); % 20 deg look up
                    q(1) =10;
                    yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, ['set target ' num2str(deg2rad(q))]);
                end
            end
        end
        
        function delete(app)
            delete(app.icubUI);
            delete(app.cntUI);
            %if ~isempty(app.icub)
            %    delete(app.icub);
            %end
        end
        
        %end
        %methods (Static)
        
        %handle keystrokes
        function keypressed(app, src, evt)
            %disp(evt.Key);
            if strcmp(evt.Key, 't')
                app.touched = 1;
                disp('touched!');
            end
            if strcmp(evt.Key,'r')
                app.OCStimulus = 1;
                disp('S1 was seen')
            end
            if strcmp(evt.Key,'g')
                app.OCStimulus = 2;
                disp('S2 was seen')
            end
        end
    end
    
end

