classdef CDemoSim < runIOSim
    %PADDLEAPP Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        gui = [];
        icubUI = [];
        stateUI = [];
        cntUI = [];
        %markerAssistant = [];
        gazeAssistant = [];
        colorTrackerRed = 0;
        colorTrackerYellow = 0;

        
        status = 0;
        PRECONNECT = -1;
        WAITFORSTART = 0;
        CONNECTING = 1;
        CALIBTACTILES = 2;
        RANDHEADMOV = 3;
        SIMPLEGAZE = 4;
        PERFORMACTION = 5;
        RUNNING = 6;
        
        cntparams = {
            'MarkerTracker', '/CCDemo/MarkerTracker:i', '/MarkerTracker/marker:o', 'input';...
            'RandomHeadMovements', '/CCDemo/RandomHeadMovements/state:o', '/RandomHeadMovements/state:i', 'output';...
            'SimpleGazeCtrl', '/CCDemo/SimpleGazeCtrl/state:o', '/SimpleGazeCtrl/command', 'output';
            'ColorTrackerRed', '/CCDemo/pf3dTracker_R/input', '/pf3dTracker_R/data:o', 'input';
            'ColorTrackerYellow', '/CCDemo/pf3dTracker_Y/input', '/pf3dTracker_Y/data:o',  'input';
            };
    end
    
    methods
        function app = CDemoSim()
            app = app@runIOSim(0);
        end
        
        function init(app)
            set(app.runbutton, 'Enable', 'off');
            set(app.runbutton, 'Visible', 'off');
            
            app.gui = CDemoGUI(app);
            app.icubUI = ICubUISimRob(app, app.gui.icubPanel);
            
            app.cntUI = YarpCntAssistant(app.cntparams, app.gui.varPanel);
            app.cntUI.hideUI();
            
            app.stateUI = StateMachineUI(app, app.gui.statePanel);
            app.stateUI.setState(app.PRECONNECT);
            
            app.gazeAssistant = SimpleGazeAssistant(app, app.gui.varPanel);
            
            app.par.maxStimulusDuration=2;
            app.colorTrackerRed =  Tracker('Red',app.cntUI.cnts.ColorTrackerRed);
            app.colorTrackerYellow =  Tracker('Yellow',app.cntUI.cnts.ColorTrackerYellow);
            
        end
        
        
        function readRobotStimuli(app)          
            %read colorTracker for stimulus
            if app.learningMode == 2
                app.colorTrackerRed.getInfos();
                app.colorTrackerYellow.getInfos();
                if app.colorTrackerRed.confidance 
                    app.OCstimuli(1) = 1;
                else
                    app.OCstimuli(1) = 0;
                end
                if app.colorTrackerYellow.confidance 
                    app.OCstimuli(2) = 1;
                else
                    app.OCstimuli(2) = 0;
                end
            else
                app.OCstimuli = zeros(app.par.nrOCStimuli,1);
            end
            
            %read stimuli (i.e. markers) used both in CC and OC scenarios
            app.CSstimuli = zeros(app.par.nrCCStimuli,1);
            marker = zeros(1,app.par.nrCCStimuli);
            [mbottle mstr] = yarpRead(app.cntUI.cnts.MarkerTracker.port);
            numDetectedMarker = str2num(mbottle.get(0).toString_c);
            for i=1:numDetectedMarker
                dmidx = str2num(mbottle.get(3*(i-1)+1).toString_c)+1;
               % disp(['Markers detected: ' num2str(dmidx)]);
                app.CSstimuli(dmidx) = 1;
            end
            
            if 0
                %TODO
            % do some low pass filtering for flickering in MarkerTracking
            app.stimulusDuration = app.stimulusDuration + min(rand(app.netCC.stream,app.par.nrCCStimuli,1) .* app.par.maxStimulusDuration .* newstimuli, app.par.maxStimulusDuration);
            app.stimulusDuration = min(max(app.stimulusDuration - app.par.samplingTime,-app.par.samplingTime),app.par.maxStimulusDuration);
            
            app.CSstimuli = (app.stimulusDuration > 0);
            end
            
            %NEEDED only for OC mode in simulation, but forget it
            if 0
            if max(app.OCstimulus) > 0;
            else
                %reset ball position to nirvana HERE?????
                ballpos = [200 200 200];
                yarpWrite(app.icubUI.icub{app.icubUI.SIMIDX}.sim.world, ['world set ssph 1 ' num2str(ballpos)]);
                yarpWrite(app.icubUI.icub{app.icubUI.SIMIDX}.sim.world, ['world set ssph 2 ' num2str(ballpos)]);
            end
            end
        end
        
        function robotRewardCC(app)
            app.performaction();
%            pause(1);
            
            app.robotRewardCC@runIOSim;
            
            yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set alpha 5');
            yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set start random');
        end
        
        function startCC(app)
            app.icubUI.setEmotion('clr');
            yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
            yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set alpha 5');
            yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set start random');
            
            app.startCC@runIOSim;
        end
        
        function startTrackingTutor(app)
            app.icubUI.setEmotion('hap');
            yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set stop');
            yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, ['set markerToTrack ' num2str(app.trackingStimulus - 1)]);
            yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 1');
        end
        
        function stopRunning(app)
            app.icubUI.setEmotion('clr');
            yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
            yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set stop');
        end
        
        function connect(app,src,evn)
            if get(src, 'Value')
                app.stateUI.setState(app.CONNECTING);
            else
                %if ~app.cntUI.isconnected() && ~app.icubUI.isconnected()
                %    app.stateUI.setState(app.PRECONNECT);
                %else
                app.stateUI.setState(app.WAITFORSTART);
                %end
            end
        end
        
        function calibtactiles(app,src,evn)
            if get(src, 'Value')
                app.stateUI.setState(app.CALIBTACTILES);
            else
                app.stateUI.setState(app.WAITFORSTART);
            end
        end
        
        function randheadmov(app,src,evn)
            if get(src, 'Value')
                app.stateUI.setState(app.RANDHEADMOV);
                yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set alpha 5');
                yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set start random');
            else
                yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set stop');
                app.stateUI.setState(app.WAITFORSTART);
            end
        end
        
        function simplegazectrl(app,src,evn)
            if get(src, 'Value')
                app.stateUI.setState(app.SIMPLEGAZE);
                yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 1');
            else
                yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
                app.stateUI.setState(app.WAITFORSTART);
            end
        end
        
        function performaction(app,src,evn)
             yarpWrite(app.cntUI.cnts.SimpleGazeCtrl.port, 'set state 0');
            %smile
            app.icubUI.setEmotion('hap');
            %gaze to person (preprogrammed)
            %TODO call RandomHeadMovement for VITE motion to desired head configuration
            %yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, ['1 ' num2str([pan tilt])]);
            q = [15 0 0 0 0 0];
            yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set alpha 10');
            yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, ['set target ' num2str(deg2rad(q))]);
            %yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, 'set stop');
            %app.icubUI.driveToPosture('head', deg2rad(q));
            if 0
                idx =find(cell2mat(app.icubUI.icubexe)==1, 1,'last');
                icub = app.icubUI.icub{idx};
                if icub.connected
                    q = rad2deg(icub.getPosture('head')); % 20 deg look up
                    q(1) =10;
                    yarpWrite(app.cntUI.cnts.RandomHeadMovements.port, ['set target ' num2str(deg2rad(q))]);
                end
            end
        end
        
        function keypressed(app, src, evt)
            %disp(evt.Key);
            if strcmp(evt.Key, 't')
                app.touched = 1;
                disp('touched!');
            end
            if strcmp(evt.Key,'p')
                app.punished = 1;
                disp('punished');
            end
            if strcmp(evt.Key,'r')
                if app.icubUI.icub{app.icubUI.SIMIDX}.connected
                    %TODO this should be detected by tracker
                    %app.currentOCStimulus = 1;
                    %assuming ball has index 0
                    ballpos = [0 0.9 0.4];
                    yarpWrite(app.icubUI.icub{app.icubUI.SIMIDX}.sim.world, ['world set ssph 1 ' num2str(ballpos)]);
                else
                    app.currentOCStimulus = 1;
                end
                disp('OC-S1 appeared');
            end
            if strcmp(evt.Key,'g')
                if app.icubUI.icub{app.icubUI.SIMIDX}.connected
                    %TODO this should be detected by tracker
                    %app.currentOCStimulus = 2;
                    %assuming ball has index 1
                    ballpos = [0 0.9 0.4];
                    yarpWrite(app.icubUI.icub{app.icubUI.SIMIDX}.sim.world, ['world set ssph 2 ' num2str(ballpos)]);
                else
                    app.currentOCStimulus = 2;
                end
                disp('OC-S2 is appeared');
            end
%             for i= 1:9
%                 if strcmp(evt.Key,num2str(i))
%                     app.newStimulus = i;
%                     disp(['CC-S' num2str(i) ' is perceived']);
%                 end
%             end
        end
        
        function delete(app)
            delete(app.icubUI);
            delete(app.cntUI);
        end
    end
    
end

