classdef SimpleGazeAssistant < handle
    %SIMPLEGAZEASSISTANT Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        app = [];
        parentup = [];
        uis = [];
        f = [];
        
        markerToTrack = [];
    end
    
    methods
        function ga = SimpleGazeAssistant(app, parentup)
            ga.app = app;
            
            if nargin < 2
                ga.f = figure('Visible','off','Position',[400,300,500,50]);
                set(ga.f,'MenuBar','none');
                set(ga.f,'Units','normalized');
                set(ga.f,'Name',class(ga));
                movegui(ga.f,'center');
                set(ga.f,'Visible','on');
                ga.parentup = ga.f;
            else
                ga.parentup = parentup;
            end
            
            ga.initUI();
        end
        
        function initUI(ga)
            ga.uis.markerToTrack = uicontrol('Style', 'pushbutton',...
                'String','marker to track','Parent',ga.parentup,'Units','normalized','Position', [0 0.5 0.25 0.5],...
                'FontSize', 12,'Visible','off');
            ga.uis.markerToTrack_val = uicontrol('Style', 'popup',...
                'String', {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9'},...
                'Parent',ga.parentup,'Units','normalized','Position', [0.25 0.5 0.25 0.5],...
                'Callback', @ga.setMarkerToTrack,'Visible','off');
            ga.uis.get = uicontrol('Style', 'Pushbutton',...
                'String', 'get',...
                'Parent',ga.parentup,'Units','normalized','Position', [0 0.25 0.25 0.25],...
                'Callback', @ga.getParams,'Visible','off');
            ga.uis.set = uicontrol('Style', 'Pushbutton',...
                'String', 'set',...
                'Parent',ga.parentup,'Units','normalized','Position', [0 0 0.25 0.25],...
                'Callback', @ga.setParams,'Visible','off');
            
            
            ga.uis.kpan = uicontrol('Style', 'pushbutton',...
                'String','k_pan','Parent',ga.parentup,'Units','normalized','Position', [0.5 0.8 0.25 0.2],...
                'FontSize', 12,'Visible','off');
            ga.uis.kpan_val = uicontrol('Style', 'edit',...
                'String','','Parent',ga.parentup,'Units','normalized','Position', [0.75 0.8 0.25 0.2],...
                'FontSize', 12,'Visible','off');
            
            ga.uis.ktilt = uicontrol('Style', 'pushbutton',...
                'String','k_tilt','Parent',ga.parentup,'Units','normalized','Position', [0.5 0.6 0.25 0.2],...
                'FontSize', 12,'Visible','off');
            ga.uis.ktilt_val = uicontrol('Style', 'edit',...
                'String','','Parent',ga.parentup,'Units','normalized','Position', [0.75 0.6 0.25 0.2],...
                'FontSize', 12,'Visible','off');
            
            ga.uis.krollcomp = uicontrol('Style', 'pushbutton',...
                'String','k_rollcomp','Parent',ga.parentup,'Units','normalized','Position', [0.5 0.4 0.25 0.2],...
                'FontSize', 12,'Visible','off');
            ga.uis.krollcomp_val = uicontrol('Style', 'edit',...
                'String','','Parent',ga.parentup,'Units','normalized','Position', [0.75 0.4 0.25 0.2],...
                'FontSize', 12,'Visible','off');
            
            ga.uis.dhome = uicontrol('Style', 'pushbutton',...
                'String','d_home','Parent',ga.parentup,'Units','normalized','Position', [0.5 0.2 0.25 0.2],...
                'FontSize', 12,'Visible','off');
            ga.uis.dhome_val = uicontrol('Style', 'edit',...
                'String','','Parent',ga.parentup,'Units','normalized','Position', [0.75 0.2 0.25 0.2],...
                'FontSize', 12,'Visible','off');
            
            ga.uis.dlowpass = uicontrol('Style', 'pushbutton',...
                'String','d_lowpass','Parent',ga.parentup,'Units','normalized','Position', [0.5 0.0 0.25 0.2],...
                'FontSize', 12,'Visible','off');
            ga.uis.dlowpass_val = uicontrol('Style', 'edit',...
                'String','','Parent',ga.parentup,'Units','normalized','Position', [0.75 0.0 0.25 0.2],...
                'FontSize', 12,'Visible','off');
        end
        
        function setMarkerToTrack(ga,src,evn)
            val = get(ga.uis.markerToTrack_val,'Value');
            str = get(ga.uis.markerToTrack_val,'String');
            if ga.app.cntUI.cnts.SimpleGazeCtrl.isconnected()
                yarpWrite(ga.app.cntUI.cnts.SimpleGazeCtrl.port, ['set markerToTrack ' str{val}]);
            end
        end
        
        function getParams(ga,src,evn)
            if ga.app.cntUI.cnts.SimpleGazeCtrl.isconnected()
                [a astring] = yarpRequest(ga.app.cntUI.cnts.SimpleGazeCtrl.port, ['get ' get(ga.uis.kpan,'String')]);
                set(ga.uis.kpan_val, 'String', char(astring));
                
                [a astring] = yarpRequest(ga.app.cntUI.cnts.SimpleGazeCtrl.port, ['get ' get(ga.uis.ktilt,'String')]);
                set(ga.uis.ktilt_val, 'String', char(astring));
                
                [a astring] = yarpRequest(ga.app.cntUI.cnts.SimpleGazeCtrl.port, ['get ' get(ga.uis.krollcomp,'String')]);
                set(ga.uis.krollcomp_val, 'String', char(astring));
                
                [a astring] = yarpRequest(ga.app.cntUI.cnts.SimpleGazeCtrl.port, ['get ' get(ga.uis.dhome,'String')]);
                set(ga.uis.dhome_val, 'String', char(astring));
                
                [a astring] = yarpRequest(ga.app.cntUI.cnts.SimpleGazeCtrl.port, ['get ' get(ga.uis.dlowpass,'String')]);
                set(ga.uis.dlowpass_val, 'String', char(astring));
            end
        end
        
        function setParams(ga,src,evn)
            if ga.app.cntUI.cnts.SimpleGazeCtrl.isconnected()
                str = get(ga.uis.kpan_val, 'String');
                [val ok] = str2num(str);
                if ok
                    yarpWrite(ga.app.cntUI.cnts.SimpleGazeCtrl.port, ['set ' get(ga.uis.kpan,'String') ' ' str]);
                end
                
                str = get(ga.uis.ktilt_val, 'String');
                [val ok] = str2num(str);
                if ok
                    yarpWrite(ga.app.cntUI.cnts.SimpleGazeCtrl.port, ['set ' get(ga.uis.ktilt,'String') ' ' str]);
                end
                
                str = get(ga.uis.krollcomp_val, 'String');
                [val ok] = str2num(str);
                if ok
                    yarpWrite(ga.app.cntUI.cnts.SimpleGazeCtrl.port, ['set ' get(ga.uis.krollcomp,'String') ' ' str]);
                end
                
                str = get(ga.uis.dhome_val, 'String');
                [val ok] = str2num(str);
                if ok
                    yarpWrite(ga.app.cntUI.cnts.SimpleGazeCtrl.port, ['set ' get(ga.uis.dhome,'String') ' ' str]);
                end
          
                str = get(ga.uis.dlowpass_val, 'String');
                [val ok] = str2num(str);
                if ok
                    yarpWrite(ga.app.cntUI.cnts.SimpleGazeCtrl.port, ['set ' get(ga.uis.dlowpass,'String') ' ' str]);
                end
            end
        end
            
        function showUI(ga)
            fields = fieldnames(ga.uis);
            for i = 1:numel(fields)
                set(ga.uis.(fields{i}),'visible','on');
            end
        end
        
        function hideUI(ga)
            fields = fieldnames(ga.uis);
            for i = 1:numel(fields)
                set(ga.uis.(fields{i}),'visible','off');
            end
        end
        
    end   
end

