classdef StateMachineUI < handle
    %GRASPASSISTANT Summary of this class goes here
    %   Detailed explanation goes here
    
    properties(SetObservable)
        app = [];
        parentup = [];
        uis = [];
        state = 0;
        f=0;
    end
    events
        StateChange;
    end
    methods
        function ga = StateMachineUI(app, parentup)
            ga.app = app;
            
            if ~exist('parentup','var')
                ga.f = figure('Visible','off','Position',[400,300,500,50]);
                set(ga.f,'MenuBar','none');
                set(ga.f,'Units','normalized');
                set(ga.f,'Name','StateMachine');
                movegui(ga.f,'center');
                set(ga.f,'Visible','on');
                ga.parentup = ga.f;
            else
                ga.parentup = parentup;
            end

            ga.initUI();            
        end
        
        function initUI(ga)
            app = ga.app;
            ga.uis.connect = uicontrol('Style', 'togglebutton',...
                'String','Connections','Min',0,'Max',1,...
                'Parent',ga.parentup,'Units','normalized','Position', [0 0.75 0.5 0.25],...
                'FontSize', 12,'Callback',@app.connect);
            ga.uis.calibtactiles = uicontrol('Style', 'togglebutton',...
                'String','Calibrate Tactiles','Min',0,'Max',1,...
                'Parent',ga.parentup,'Units','normalized','Position', [0 0.5 0.5 0.25],...
                'FontSize', 12,'Callback',@app.calibtactiles);
            ga.uis.randheadmov = uicontrol('Style', 'togglebutton',...
                'String','RandomHeadMovements','Min',0,'Max',1,...
                'Parent',ga.parentup,'Units','normalized','Position', [0 0.25 0.5 0.25],...
                'FontSize', 12,'Callback',@app.randheadmov);
            ga.uis.simplegazectrl = uicontrol('Style', 'togglebutton',...
                'String','SimpleGazeCtrl','Min',0,'Max',1,...
                'Parent',ga.parentup,'Units','normalized','Position', [0 0 0.5 0.25],...
                'FontSize', 12,'Callback',@app.simplegazectrl);
            ga.uis.performaction = uicontrol('Style', 'pushbutton',...
                'String','Perform Action',...
                'Parent',ga.parentup,'Units','normalized','Position', [0.5 0.5 0.5 0.5],...
                'FontSize', 12,'Callback',@app.performaction);
            ga.uis.running = uicontrol('Style', 'togglebutton',...
                'String','Run System','Min',0,'Max',1,...
                'Parent',ga.parentup,'Units','normalized','Position', [0.5 0 0.5 0.5],...
                'FontSize', 12,'Callback',@app.running);
        end
        
        
        function setState(ga, state)
           ga.app.status = state;
           switch state
               case ga.app.PRECONNECT
                   fields = fieldnames(ga.uis);
                   for i=1:numel(fields)
                       set(ga.uis.(fields{i}),'Enable','off');
                       set(ga.uis.(fields{i}),'Value',0);
                   end
                   set(ga.uis.connect,'Enable','on');
                   set(ga.uis.connect,'Value',0);
                  
                   ga.app.cntUI.hideUI();
                   
               case ga.app.CONNECTING
                   fields = fieldnames(ga.uis);
                   for i=1:numel(fields)
                       set(ga.uis.(fields{i}),'Enable','off');
                       set(ga.uis.(fields{i}),'Value',0);
                   end
                   set(ga.uis.connect,'Enable','on');
                   set(ga.uis.connect,'Value',1);
                   
                   ga.app.cntUI.showUI();
                   
               case ga.app.WAITFORSTART
                   ga.app.cntUI.hideUI();
                   set(ga.uis.connect,'Enable','on');
                   set(ga.uis.connect,'Value',0);
                   ga.app.gazeAssistant.hideUI();
                   if strcmp(class(ga.app), 'CDemo')
                    ga.app.tactileAssistant.hideUI();
                   end
                   
                   if isfield(ga.app.cntUI.cnts, 'TactilesRight') && ga.app.cntUI.cnts.TactilesRight.isconnected()
                       set(ga.uis.calibtactiles, 'Enable', 'on');
                   else
                       set(ga.uis.calibtactiles, 'Enable', 'off');
                       set(ga.uis.calibtactiles, 'Value', 0);
                   end
                   
                   if ga.app.cntUI.cnts.RandomHeadMovements.isconnected()
                       set(ga.uis.randheadmov, 'Enable', 'on');
                   else
                       set(ga.uis.randheadmov, 'Enable', 'off');
                       set(ga.uis.randheadmov, 'Value', 0);
                   end
                   
                   if ga.app.cntUI.cnts.SimpleGazeCtrl.isconnected()
                       set(ga.uis.simplegazectrl, 'Enable', 'on');
                   else
                       set(ga.uis.simplegazectrl, 'Enable', 'off');
                       set(ga.uis.simplegazectrl, 'Value', 0);
                   end
                   
                   if ga.app.icubUI.isconnected() && ga.app.cntUI.cnts.RandomHeadMovements.isconnected()
                       set(ga.uis.performaction, 'Enable', 'on');
                       %ga.app.icubUI.setEmotion(ga.app.WAITFORSTART+1);
                   else
                       set(ga.uis.performaction, 'Enable', 'off');
                   end
                   
                   if ga.app.icubUI.isconnected() && ga.app.cntUI.isconnected()
                       set(ga.uis.running, 'Enable', 'on');
                   else
                       set(ga.uis.running, 'Enable', 'off');
                       set(ga.uis.running, 'Value', 0);
                   end
                   
               case ga.app.CALIBTACTILES
                   fields = fieldnames(ga.uis);
                   for i=1:numel(fields)
                       set(ga.uis.(fields{i}),'Enable','off');
                       set(ga.uis.(fields{i}),'Value',0);
                   end
                   set(ga.uis.calibtactiles,'Enable','on');
                   set(ga.uis.calibtactiles,'Value',1);
                   
                   ga.app.tactileAssistant.showUI();
                   
               case ga.app.RANDHEADMOV
                   fields = fieldnames(ga.uis);
                   for i=1:numel(fields)
                       set(ga.uis.(fields{i}),'Enable','off');
                       set(ga.uis.(fields{i}),'Value',0);
                   end
                   set(ga.uis.randheadmov,'Enable','on');
                   set(ga.uis.randheadmov,'Value',1);
                   
                   %ga.app.randheadmovAssistant.showUI();
                   
               case ga.app.SIMPLEGAZE
                   fields = fieldnames(ga.uis);
                   for i=1:numel(fields)
                       set(ga.uis.(fields{i}),'Enable','off');
                       set(ga.uis.(fields{i}),'Value',0);
                   end
                   set(ga.uis.simplegazectrl,'Enable','on');
                   set(ga.uis.simplegazectrl,'Value',1);
                   
                   ga.app.gazeAssistant.showUI();

               case ga.app.RUNNING
                   fields = fieldnames(ga.uis);
                   for i=1:numel(fields)
                       set(ga.uis.(fields{i}),'Enable','off');
                       set(ga.uis.(fields{i}),'Value',0);
                   end
                   set(ga.uis.running,'Enable','on');
                   set(ga.uis.running,'Value',1);
                   
                   %ga.app.simplegazeAssistant.showUI();
           end
        end
        
    end
end
    
