classdef trackerUIRedYellow < handle
    %CTRLGUI Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        tracker = {[], []};
        trackerexe = {1, 0};
        trackernames = {'Left', 'Right'};
        
        
        app = [];
        f = [];
        parentup = [];
        uis = [];
        
        LEFTIDX = 1;
        RIGHTIDX = 2;
    end
    
    methods
        function ui = trackerUILeftRight(app, parentup)
            ui.app = app;
            if nargin < 2
                ui.f = figure('Visible','off','Position',[200,150,400,350]);
                set(ui.f,'MenuBar','none');
                set(ui.f,'Units','normalized');
                set(ui.f,'Name','trackerUILeftRight');
                movegui(ui.f,'center');
                set(ui.f,'Visible','on');
                ui.parentup = ui.f;
            else
                ui.parentup = parentup;
            end
            
            ui.initUI();
        end
        
        function initUI(ui)
            %%%%%%%%% trackerYarp %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            ui.uis.trackerLeftUIPanel = uipanel('Title', 'Left',...
                'Parent',ui.parentup,...
                'Units','normalized',...
                'Position', [0 0 0.5 1],...
                'FontSize', 12, 'BackgroundColor', 'white');
            ui.uis.trackerRightUIPanel = uipanel('Title', 'Right',...
                'Parent',ui.parentup,...
                'Units','normalized',...
                'Position', [0.5 0 0.5 1],...
                'FontSize', 12, 'BackgroundColor', 'white');
            ui.uis.connectRight = uicontrol('Style', 'pushbutton',...
                'String','connect',...
                'Parent',ui.uis.trackerRightUIPanel,'Units','normalized','Position', [0 0.5 1 0.5],...
                'FontSize', 12,'Callback',@ui.connect);
            ui.uis.connectLeft = uicontrol('Style', 'pushbutton',...
                'String','connect',...
                'Parent',ui.uis.trackerLeftUIPanel,'Units','normalized','Position', [0 0.5 1 0.5],...
                'FontSize', 12,'Callback',@ui.connect);
            ui.uis.disconnectLeft = uicontrol('Style', 'pushbutton',...
                'String','disconnect',...
                'Parent',ui.uis.trackerLeftUIPanel,'Units','normalized','Position', [0 0 1 0.5],...
                'FontSize', 12,'Callback',@ui.disconnect);
            ui.uis.disconnectRight = uicontrol('Style', 'pushbutton',...
                'String','disconnect',...
                'Parent',ui.uis.trackerRightUIPanel,'Units','normalized','Position', [0 0 1 0.5],...
                'FontSize', 12,'Callback',@ui.disconnect);
            %             ui.uis.exeLeft = uicontrol('Style', 'togglebutton',...
            %                 'String','execute','Min',0,'Max',1,'Value',ui.trackerexe{ui.LEFTIDX},...
            %                 'Parent',ui.uis.trackerLeftUIPanel,'Units','normalized','Position', [0 0 1 0.33],...
            %                 'FontSize', 12,'Callback',@ui.execute);
            %             ui.uis.exeRight = uicontrol('Style', 'togglebutton',...
            %                 'String','execute','Min',0,'Max',1,'Value',ui.trackerexe{ui.RIGHTIDX},...
            %                 'Parent',ui.uis.trackerRightUIPanel,'Units','normalized','Position', [0 0 1 0.33],...
            %                 'FontSize', 12,'Callback',@ui.execute);
            %
            
            
            
        end
        function execute(ui,src,evn)
            idx = -1;
            if src == ui.uis.exeLeft
                idx = ui.LEFTIDX;
            end
            if src == ui.uis.exeRight
                idx = ui.RIGHTIDX;
            end
            if idx <= 0 || idx > 2
                return;
            end
            
            ui.trackerexe{idx} = get(src, 'Value');
        end
        
        function connect(ui,src,evn)
            idx = -1;
            if src == ui.uis.connectLeft
                idx = ui.LEFTIDX;
                [status,result] = system(['yarp connect /'  ui.app.icubUI.robotname '/cam/left /pf3dTracker_L/video:i'  ]);
            end
            if src == ui.uis.connectRight
                idx = ui.RIGHTIDX;
                [status,result] = system(['yarp connect /'  ui.app.icubUI.robotname '/cam/right /pf3dTracker_R/video:i'  ]);
            end
            if idx <= 0 || idx > 2
                return;
            end
            
            if ~isempty(ui.tracker{idx})
                ui.tracker{idx}.close();
                ui.tracker{idx} = [];
            end
            
            
            ui.tracker{idx} = Tracker(ui.trackernames{idx});
            ui.tracker{idx}.open();
            
            
            
            
            if ui.tracker{idx}.connected
                set(src, 'Enable', 'off');
            else
                ui.tracker{idx}.close();
                ui.tracker{idx} = [];
            end
            
            isconnected = 0;
            for i=1:2
                if ~isempty(ui.tracker{i})
                    isconnected = ui.tracker{i}.connected;
                end
            end
            if isconnected
                ui.app.stateUI.setState(ui.app.WAITFORSTART);
            else
                ui.app.stateUI.setState(ui.app.PRECONNECT);
            end
        end
        
        function disconnect(ui,src,evn)
            idx = -1;
            if src == ui.uis.disconnectLeft
                idx = ui.LEFTIDX;
            end
            if src == ui.uis.disconnectRight
                idx = ui.RIGHTIDX;
            end
            if idx <= 0 || idx > 2
                return;
            end
            
            if ~isempty(ui.tracker{idx})
                if ui.tracker{idx}.connected && ~isempty(ui.tracker{idx})
                    ui.tracker{idx}.close();
                    disp(['disconnected '  ui.tracker{idx}.trackername ' Tracker']);
                else
                    ui.tracker{idx}.close();
                    
                end
            end
            
            ui.tracker{idx} = [];
            
            if idx == ui.LEFTIDX
                set(ui.uis.connectLeft, 'Enable', 'on');
            end
            if idx == ui.RIGHTIDX
                set(ui.uis.connectRight, 'Enable', 'on');
            end
            
            
        end
        
        function b = isconnected(ui)
            b = zeros(1, 2);
            for i=1:2
                if ~isempty(ui.tracker{i})
                    b(i) = ui.tracker{i}.connected;
                end
            end
            b = all(b);
        end
        
        function [pixels confidance]= getInfo(ui)
            pixels=[];
            confidance = 0;
            if ui.isconnected
                ui.tracker{1}.getInfos;
                ui.tracker{2}.getInfos;
                pixels = [ui.tracker{1}.pixelCoord'; ui.tracker{2}.pixelCoord'];
                if ui.tracker{1}.confidance && ui.tracker{2}.confidance
                    confidance = 1;
                end
            
            end
        end
        
        
    end
    
    
    
end

