%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%                      HEAD CONTROLLER CLASS               
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
classdef ICubHeadCtrl < handle
    %TO DO Class description
    
    properties
        icub;% Icub object
        portLeft;
        portRight;
        connected = 0;
        
        velctrl = 1; %flag for switching velocity and position ctrl
        Pleft = [-1 -1];
        Pright = [-1 -1];
        confidence = 0;
        confidenceLeft = 0;
        confidenceRight = 0;
        dq = [];
        tStart = 0;
        tElapsed = 0;
    end
    
    
    
    methods
        function ctrl = ICubHeadCtrl(icub)
            ctrl.icub = icub;
        end %ICubHeadCtrl()
        
        
        function open(ctrl)
            if ctrl.connected
                return;
            end
            
            getYarped();
            
            ctrl.connected = 1;
            
            % Open a port  for the left eye
            portnameLeft = '/client/tracker/left';
            ctrl.portLeft = yarp.Port;
            a = ctrl.portLeft.open(portnameLeft);
            if ~a
                ctrl.connected = 0;
                warning(['could not open port ' portnameLeft]);
                return;
            end
            system(['yarp connect /blobTrackerLEFT/leftEye/targetCoordinates:o ' portnameLeft]);
            pause(0.01);
            
            % Open a port  for the right eye
            portnameRight = '/client/tracker/right';
            ctrl.portRight = yarp.Port;
            a = ctrl.portRight.open(portnameRight);
            if ~a
                ctrl.connected = 0;
                warning(['could not open port ' portnameRight]);
                return;
            end
            system(['yarp connect /blobTrackerRIGHT/rightEye/targetCoordinates:o ' portnameRight]);
            pause(0.01);
            
            ctrl.tStart = tic;
        end
        
        function close(ctrl)
            if ~ctrl.connected
                return;
            end
            
            ctrl.portLeft.close();
            ctrl.portRight.close();

            ctrl.connected = 0;
            disp('disconnected ICubHeadCtrl');
        end
        
        function delete(ctrl)
            ctrl.close();
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       
         
          % This function calculates the jacobian of the waist and the head
          % for both eyes (the head of iCub is version 1)
          
        function [H_0L, H_0R, Jl, Jr] = WaistHeadFwdKin_HeadV2(ctrl, wtheta,htheta)
            
            wtheta0 = wtheta(3);
            wtheta1 = wtheta(2);
            wtheta2 = wtheta(1);
            
            %Reference frames attached to links
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %               Waist                       *
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
           G_01=evalDHMatrix(     32,        0,     pi/2,                wtheta0);
            G_12=evalDHMatrix(      0,     -5.5,     pi/2,                wtheta1 - pi/2);
            G_23=evalDHMatrix(      2.31,   -193.3,    -pi/2,                wtheta2 - pi/2 );
             
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %               Head                        *
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            
            theta0 = htheta(1);
            theta1 = htheta(2);
            theta2 = htheta(3);
            theta3 = htheta(4);
            theta4 =  (- htheta(6) + htheta(5))/2; %conversion from vergence and version to right eye pan
            theta5 =  (htheta(6) + htheta(5))/2; %conversion from vergence and version to left eye pan
            
             G_34=evalDHMatrix(    33,     0,     pi/2,      theta0 + pi/2);
             G_45=evalDHMatrix(     0,      1,    -pi/2,      theta1 - pi/2 );
             G_56=evalDHMatrix(   -54,    82.5,    -pi/2,      theta2 + pi/2) ;
             
             
             G_67=evalDHMatrix(     0,     34,    -pi/2,      theta3);    %right eye
             G_78=evalDHMatrix(     0,      0,     pi/2,      theta4 - pi/2); %right eye
             
             Gl_67=evalDHMatrix(    0,    -34,    -pi/2,      theta3); %left eye
             Gl_78=evalDHMatrix(    0,      0,     pi/2,      theta5-pi/2);% left eye
             

             
            H_0L = G_01*G_12*G_23*G_34*G_45*G_56*Gl_67*Gl_78;
            H_0R = G_01*G_12*G_23*G_34*G_45*G_56*G_67*G_78;
            
            P10= -G_01(1:3, 4);% vector from frame 1 to frame 0
            vo1 = cross(P10, [0 0 1]');
            vo1 = [vo1; 0];
            vo1 = G_01\vo1; % Express vector from frame 0 to frame 1
            vo1 = vo1(1:3);
            
            P21= -G_12(1:3, 4);% vector from frame 2 to frame 1
            vo2 = cross(P21, [0 0 1]');
            vo2 = [vo2; 0];
            vo2 = G_12\vo2;
            vo2 = vo2(1:3);
            
            P32= -G_23(1:3, 4);% vector from frame 2 to frame 1
            vo3 = cross(P32, [0 0 1]');
            vo3 = [vo3; 0];
            vo3 = G_23\vo3;
            vo3 = vo3(1:3);
            
            P43= -G_34(1:3, 4);% vector from frame 2 to frame 1
            vo4 = cross(P43, [0 0 1]');
            vo4 = [vo4; 0];
            vo4 = G_34\vo4;
            vo4 = vo4(1:3);
            
            P54= -G_45(1:3, 4);% vector from frame 2 to frame 1
            vo5 = cross(P54, [0 0 1]');
            vo5 = [vo5; 0];
            vo5 = G_45\vo5;
            vo5 = vo5(1:3);
            
            P65= -G_56(1:3, 4);% vector from frame 2 to frame 1
            vo6 = cross(P65, [0 0 1]');
            vo6 = [vo6; 0];
            vo6 = G_56\vo6;
            vo6 = vo6(1:3);
            
            % Right eye
            P76= -G_67(1:3, 4);% vector from frame 2 to frame 1
            vo7 = cross(P76, [0 0 1]');
            vo7 = [vo7; 0];
            vo7 = G_67\vo7;
            vo7 = vo7(1:3);
            
            P87= -G_78(1:3, 4);% vector from frame 2 to frame 1
            vo8 = cross(P87, [0 0 1]');
            vo8 = [vo8; 0];
            vo8 = G_78\vo8;
            vo8 = vo8(1:3);
            
            % left eye
            Pl76= -Gl_67(1:3, 4);% vector from frame 2 to frame 1
            vol7 = cross(Pl76, [0 0 1]');
            vol7 = [vol7; 0];
            vol7 = Gl_67\vol7;
            vol7 = vol7(1:3);
            
            Pl87= -Gl_78(1:3, 4);% vector from frame 2 to frame 1
            vol8 = cross(Pl87, [0 0 1]');
            vol8 = [vol8; 0];
            vol8 = Gl_78\vol8;
            vol8 = vol8(1:3);
            
            %
            % Express rotationnal velocity from frame i-1 to frame i
            %
            
            w = [ 0 0 1 0]';
            
            w10 = G_01\w;
            w21 = G_12\w;
            w32 = G_23\w;
            w43 = G_34\w;
            w54 = G_45\w;
            w65 = G_56\w;
            
            w76 = G_67\w;
            w87 = G_78\w;
            
            wl76 = Gl_67\w;
            wl87 = Gl_78\w;
            
            w10 = w10(1:3);
            w21 = w21(1:3);
            w32 = w32(1:3);
            w43 = w43(1:3);
            w54 = w54(1:3);
            w65 = w65(1:3);
            w76 = w76(1:3);
            w87 = w87(1:3);
            wl76 = wl76(1:3);
            wl87 = wl87(1:3);
            
            
            % Calculate the Jacobian for the right eye
            Jr =  ctrl.adjointHomogenMat(G_01)*[w10; vo1];
            
            Jr = [ Jr ctrl.adjointHomogenMat(G_01*G_12)*[w21; vo2] ];
            Jr = [ Jr ctrl.adjointHomogenMat(G_01*G_12*G_23)*[w32; vo3] ];
            Jr = [ Jr ctrl.adjointHomogenMat(G_01*G_12*G_23*G_34)*[w43; vo4] ];
            Jr = [ Jr ctrl.adjointHomogenMat(G_01*G_12*G_23*G_34*G_45)*[w54; vo5] ];
            Jr = [ Jr ctrl.adjointHomogenMat(G_01*G_12*G_23*G_34*G_45*G_56)*[w65; vo6] ];
            Jr = [ Jr ctrl.adjointHomogenMat(G_01*G_12*G_23*G_34*G_45*G_56*G_67)*[w76; vo7] ];
            Jr = [Jr zeros(6, 1) ];
            Jr = [ Jr ctrl.adjointHomogenMat(H_0R)*[w87; vo8] ];
            
            
            % Calculate the Jacobian for the left eye
            Jl = ctrl.adjointHomogenMat(G_01)*[w10; vo1];
            
            Jl = [Jl ctrl.adjointHomogenMat(G_01*G_12)*[w21; vo2]];
            Jl = [Jl ctrl.adjointHomogenMat(G_01*G_12*G_23)*[w32; vo3]];
            Jl = [Jl ctrl.adjointHomogenMat(G_01*G_12*G_23*G_34)*[w43; vo4]];
            Jl = [Jl ctrl.adjointHomogenMat(G_01*G_12*G_23*G_34*G_45)*[w54; vo5]];
            Jl = [Jl ctrl.adjointHomogenMat(G_01*G_12*G_23*G_34*G_45*G_56)*[w65; vo6]];
            
            Jl = [Jl ctrl.adjointHomogenMat(G_01*G_12*G_23*G_34*G_45*G_56*Gl_67)*[wl76; vol7]];
            Jl = [Jl ctrl.adjointHomogenMat(H_0L)*[wl87; vol8]];
            Jl = [Jl zeros(6, 1)];
            
            
        end %WaistHeadFwdKin_HeadV2
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        % Calculate the Denavit Hartenberg matrix given the four parameters
        
        function G = evalDHMatrix(a, d, alph, thet)
            
            G = [ cos(thet), -sin(thet)*cos(alph),  sin(thet)*sin(alph), cos(thet)*a
                  sin(thet),  cos(thet)*cos(alph), -cos(thet)*sin(alph), sin(thet)*a
                  0,          sin(alph),            cos(alph),                    d
                  0,                    0,                    0,                  1];
            
        end % evalDHMatrix
        
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %
        %  This function calculates the adjoint transformation
        %  of an homogeneous matrix (see page 55 of
        %  "A mathematical introduction to robotic manipulation",R.Murray,1994)
        %
        
        function [Ad] = adjointHomogenMat(ctrl, H)
            R = H(1:3, 1:3);
            p = H(1:3, 4);
            
            Ad = [R zeros(3,3); ctrl.skewSym(p)*R R];
            
        end % adjointHomogenMat
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        % Calculate the matrix F giving the relation between the joints
        % velocity (dq) and the target velocity (dx) : dx = F*dq (see the
        % paper 'Vision Based Motion Control for a Humanoid Head'by Visser,
        % Carloni and Stramigioli
        
        function F = jointVelocityToTargetVelocity(ctrl, wtheta, htheta, Pleft, Pright)

            f = 257.34;% focal lemgth (depth) of cameras
     
            [H_0L, H_0R, Jl, Jr] = ctrl.WaistHeadFwdKin_HeadV2(wtheta,htheta);
            
            
            Adl = ctrl.adjointHomogenMat(inv(H_0L));
            Adr = ctrl.adjointHomogenMat(inv(H_0R));
            
            Pl = ctrl.skewSym([ Pleft(1) Pleft(2) f]);% left eye
            Pr = ctrl.skewSym([ Pright(1) Pright(2) f]);% right eye
            
            
            Fl = [1 0 0; 0 1 0]*[Pl, -eye(3)]*Adl*Jl;% left eye
            Fr = [1 0 0; 0 1 0]*[Pr, -eye(3)]*Adr*Jr;% right eye
            
            F = [Fl; Fr];
        end %jointVelocityToTargetVelocity
        
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
        %
        % This function calculates the skew symetric 3x3 matrix P obtained from
        % vector p such that cross(p,z) = P*z, where cross is the cross product,
        % and  z a vector.
        %
        
        function P = skewSym(ctrl, p)
            
            P = [ 0 -p(3) p(2); p(3) 0 -p(1); -p(2) p(1) 0];
            
        end %skewSym
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %
        % This function calculates pF, the weighted generalized inverse of F
        % (cf. jointVelocityToTargetVelocity to know more about F)
        %
        
        function [pF] = pseudoInv(ctrl, F)
           
            if ctrl.velctrl
                % quite good(for velocity control)
                m = [100000 500000 100000  1 1 1 0.005 0.04 0.04];
            else
                m = [100000 500000 100000  0.02 0.4 0.4 0.005 0.04 0.04];% position control
            end

            % M is a matrix that defines a metric. It allows to change the importance of
            % joints. The higher the weights are, the less important the joint velocity is.
            M = diag(m);
            
            % and the pseudoinverse of F...
            pF = inv(M)*F'*inv(F*inv(M)*F');
        end  % pseudoInv
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
        %
        %  This function calculates the desired joints velocity to servo the head.
        %  It corresponds to a solution of the inversion of : dx = F*dq
        %
        
        function dq =  desiredJointsVelocity(ctrl, x, F, pseudoInvF, q)
           
            % Proportionnal gain
            if ctrl.velctrl
                k1 = 0.9*[0.8 0.5 0.8 0.5]; % for Velocity control
            else
                    k1 = 9*[0.8 0.5 0.8 0.5];% position control
            end
           
            %k1 = 5*k1;
            %k1 = (9/8)*[1 0.5 1 0.5];
            %k1 = 2*k1; %3 also OK, 5 is max, 10 damped oscillation, 20 still "stable" oscillation
            %k1 = 2 * k1;
            K1= diag(k1);
            
            
            % Calculate the desired arget velocity. The target must be at
            % the center of camera frame,i.e. at x = y = 0.
            dx = -K1*x ;
            
            % W is a diagonal matrix. Tuning its coefficients allow to
            % change the final gazing position. A non zero coefficient
            % implies that the angle of the corresponding link must be zero
            % at the END (and ONLY at the end) of gazing
            
            if ctrl.velctrl
                w = 9*[0 0 0 0 0 0 10 0.4 0];% good for velocity control
            else
                w = 2*9*[0 0 0 0 0 0 30 0.4 0];%  for position control
            end
            
            W = diag(w);
            z = -W*(q);
            
            % and the desired joint velocity...
            dq = pseudoInvF*dx + ( eye(9)- pseudoInvF*F )*z;
           
        end %desiredJointsVelocity
        
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        % Calculate the roll angle of the neck (theta5), given tilt and pan, so that
        % head roll may be zero
        
        function theta5 = rollCompensation(ctrl, theta4, theta6)
            
            theta5 = asin( (sin(theta6)*sin(theta4))/(cos(theta6)*cos(theta4)) );
            
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        % Send the desired joint velocity calculated by the function
        % 'desiredJointsVelocity' to the robot
        
        function [dq] = setSpeed(ctrl, dq, q)
             % Angle saturation
             dq(8) = ( dq(8) + dq(9) )/2; % version
             dq(9) = ( dq(8) - dq(9) )/2; % vergence

            %ctrl.icub.driveWithVelocity('torso', dq(1:3));
            
            % Roll Compensation
            desiredRoll = ctrl.rollCompensation(q(4), q(6));
            dq(5) = 0.2 * (desiredRoll - q(5));% for Velocity control
            % dq(5) = 0.05 * (desiredRoll - q(5));% for position control
            
            ctrl.icub.driveWithVelocity('head', dq(4:9));
            %dq'

        end %setSpeed


        % Send the desired joint velocity calculated by the function
        % 'desiredJointsVelocity' to the robot via PositionCtrler
        
        function [dq] = setPos(ctrl, dq, q, tElapsed)
             % Angle saturation
             dq(8) = ( dq(8) + dq(9) )/2; % version
             dq(9) = ( dq(8) - dq(9) )/2; % vergence
            
            % Roll Compensation
            desiredRoll = ctrl.rollCompensation(q(4), q(6));
            dq(5) = 0.05 * (desiredRoll - q(5));% for position control
                      
            %qt = ctrl.icub.getPosture('torso');
            %ctrl.icub.driveToPosture('torso', qt + dq(1:3)');
      
            % POSITION CONTROL
            qh = ctrl.icub.getPosture('head');
            ctrl.icub.driveToPosture('head', qh + 0.01*dq(4:9)');
            
            %with low-pass filtering
            %qh = ctrl.icub.getPosture('head');
            %qhnew = qh + tElapsed*dq(4:9)';
            %delta = 0.1;
            %qhneck = (1-delta) * qh(1:3) + delta * qhnew(1:3);
            %ctrl.icub.driveToPosture('head', [qhneck qhnew(4:6)]);
        end %setSpeedPos
        
        function [conf P] = updateTrackerInfo(ctrl)
            if ~ctrl.connected
                return;
            end
            
            ctrl.tStart = tic;
            
            % Get the target cordinates from the trackers of left and right eyes
            bLeft = yarp.Bottle;
            ctrl.portLeft.read(bLeft, boolean(false));
            pause(0.01);
            bRight = yarp.Bottle;
            ctrl.portRight.read(bRight, boolean(false));
            pause(0.01);
            
            ctrl.confidenceLeft = bLeft.get(3).asInt();
            ctrl.confidenceRight = bRight.get(3).asInt();

            %if the confidence of the trackers is superior to 0.5 -> SERVOING
            
            if ctrl.confidenceLeft > 0.5 && ctrl.confidenceRight > 0.5
                ctrl.confidence = 1;
                
                ctrl.Pleft(1) = bLeft.get(1).asInt();
                ctrl.Pleft(2) = bLeft.get(2).asInt();
                ctrl.Pright(1) = bRight.get(1).asInt();
                ctrl.Pright(2) = bRight.get(2).asInt();
            else
                ctrl.confidence = 0;
                ctrl.Pleft = [-1 -1];
                ctrl.Pright = [-1 -1];
                %fprintf('I do not know where the target is \n');
            end
            
            conf = ctrl.confidence;
            P = [ctrl.Pleft ctrl.Pright];
        end
        
        function [wtheta, htheta] = updateCtrl(ctrl)
            if 0
            if isa(ctrl.icub, 'ICub')
                if ~ctrl.icub.connected
                    return;
                end
            elseif isa(ctrl.icub, 'ICubUISimRob')
                %if ~ctrl.icub.isconnected()
                %    return;
                %end
            else
                return;        
            end
            end
            
            if ctrl.confidence
                % Get the cuurrent joint angles of iCub
                %wtheta = ctrl.icub.getPosture('torso');
                wtheta = [0 0 0];
                htheta = ctrl.icub.getPosture('head');
                x = [ctrl.Pleft ctrl.Pright]';
 
                F = ctrl.jointVelocityToTargetVelocity(wtheta, htheta, ctrl.Pleft, ctrl.Pright);
                pF = ctrl.pseudoInv(F);
                ctrl.dq =  ctrl.desiredJointsVelocity(x, F, pF, [wtheta htheta]');
        
                ctrl.tElapsed = toc(ctrl.tStart);
                if ctrl.velctrl
                    ctrl.dq = ctrl.setSpeed(ctrl.dq, [wtheta htheta]');
                else
                    ctrl.dq = ctrl.setPos(ctrl.dq, [wtheta htheta]',  ctrl.tElapsed);
                end
            else
                delta = 0.01;
                
                wtheta = [0 0 0];
                htheta = ctrl.icub.getPosture('head');
                
                ctrl.dq = [];
                ctrl.icub.driveToPosture('head', (1-delta)*htheta );
                %ctrl.icub.driveToPosture('torso', zeros(1,3));
                pause(0.01);
            end
        end

    end%methods
    
end%class
