classdef ICub < handle
    %ICUB Summary of this class goes here
    %   Detailed explanation goes here
    %TODO home posture, check units
    
    properties
        robotname = 'icubSim';
        bodyparts = {'right_arm', 'left_arm', 'torso', 'head'};
        %bodyparts = {'head'};
        
        connected = 0;
        sim = [];
        pos = [];
        enc = [];
        mod = [];
        trq = [];
        imp = [];
        pid = [];
        
        dbgoptions = [];
        dbgdevices = [];
        dbglocalPorts = [];
        dbg = [];
        
        jimp = [];
        hjimp = [];
        
        localPorts = {};
        remotePorts = {};
        options = {};
        devices = {};
        
        qlim = [];
        qhome = [];
        xhome = [];
        xlim = [];
        portEmotions=[];
        %emotions={'sad','hap', 'evi', 'shy', 'cun', 'sur', 'ang', 'neu'};
        %emotions={'sad','hap', 'evi', 'cun', 'cun', 'sur', 'ang', 'neu'};
        emotions={'hap','neu', 'sur', 'neu', 'neu', 'hap', 'ang', 'cun'};
        
        views = {'0 0 -0.3 0 0', '0 0.2 -1 90 0', ...
            '-0.011133 0.20579 -0.589672 27.399933 -1.0', ...
            '-0.2 -0.2 -0.5 -20 0', '0.1 -0.2 -0.5 -20 -30', ...
            '-0.8 0 -1 0 90', '0.8 0 -1 0 -90', ...
            '-0.6 0 -0.5 0 60', '0.6 0 -0.5 0 -60'};
        
                
        CABLE_WARNING=10;
        b =[];
        A=[];
        
        
        
    end
    
    methods
        
        function icub = ICub(robotname, bodyparts)
            if nargin > 0
                icub.robotname = robotname;
            end
            if nargin > 1
                icub.bodyparts = bodyparts;
            end
            
            thisScope = ['/icubctrlmatlab/' icub.robotname '/'];
            for i=1:length(icub.bodyparts)
                icub.localPorts{i} = [thisScope filesep icub.bodyparts{i}];
                icub.remotePorts{i} = [filesep icub.robotname filesep icub.bodyparts{i}];
            end
            
            tightJointLimits = 0;
            
            bpid = icub.getBPID('right_arm');
            if bpid > 0
                icub.qlim{bpid} = [
                    deg2rad([-95.5 10]);
                    deg2rad([0 160]);
                    deg2rad([-37 80]);
                    deg2rad([15 106]);
                    deg2rad([-90 90]);
                    deg2rad([-90 0]);
                    deg2rad([-20 40]);
                    ...
                    deg2rad([0 60]);
                    deg2rad([10 90]);
                    deg2rad([0 90]);
                    deg2rad([0 180]);
                    deg2rad([0 90]);
                    deg2rad([0 180]);
                    deg2rad([0 90]);
                    deg2rad([0 180]);
                    deg2rad([0 270]);
                    ];
                if tightJointLimits
                    icub.qlim{bpid} = [
                        deg2rad([-80 0]);
                        deg2rad([15 120]);
                        deg2rad([-20 60]);
                        deg2rad([15 85]);
                        deg2rad([-30 30]);
                        deg2rad([-50 -5]);
                        deg2rad([-20 20]);
                        ...
                        deg2rad([-180 180]);
                        deg2rad([-180 180]);
                        deg2rad([-180 180]);
                        deg2rad([-180 180]);
                        deg2rad([-180 180]);
                        deg2rad([-180 180]);
                        deg2rad([-180 180]);
                        deg2rad([-180 180]);
                        deg2rad([-180 180]);
                        ];
                end
                icub.qhome{bpid} = deg2rad([-30 30 0 45 0 0 40  15 30 0 0 0 0 0 0]);
                %icub.qhome{bpid} = deg2rad([-25 20 15 50 0 0 0  15 30 0 0 0 0 0 0]);  %sim
                icub.xhome{bpid} = icub.fkineRA([0 0 0], icub.qhome{bpid}, 'rad');
                icub.xlim{bpid} = [-0.33 0.25; -0.11 0.45; -0.16 0.45];
                %icub.jimp{bpid} = [0.2 0; 0.2 0; 0.2 0; 0.2 0; 0.1 0]; %soft
                %%icub.jimp{bpid} = [0.1 0.001; 0.1 0.001; 0.1 0.001; 0.1 0.001; 0.05 0. ]; %soft modified
                icub.jimp{bpid} = [0.1 0.01; 0.1 0.01; 0.1 0.01; 0.1 0.01; 0.05 0. ]; %soft modified
                %icub.jimp{bpid} = [0.4 0.03; 0.4 0.03; 0.4 0.03; 0.2 0.01; 0.2 0]; %medium
                %icub.hjimp{bpid} = [0.6 0.06; 0.6 0.06; 0.6 0.06; 0.3 0.02; 0.2 0]; %hard
                icub.hjimp{bpid} = [0.6 0.03; 0.6 0.03; 0.6 0.03; 0.3 0.01; 0.2 0]; %hard modified
                %icub.jimp{bpid} = [0.4 0.02; 0.4 0.02; 0.4 0.02; 0.2 0.01; 0.2 0]; %own1
                %icub.jimp{bpid} = [0.25 0; 0.25 0; 0.25 0; 0.25 0; 0.1 0]; %own2
                
                
               icub.b = [-347.0; -366.57;-112.42;-66.6;-213.3;-96.0;-5.0;0;-195.0; -90.0; -90.0];
                c= 1.71;
                icub.A =[   c, -c,  0;...
                            c,  -c,  -c;...
                            -c,  c,  c;...
                            0,  1,  1;...
                            0, -1,  -1;...
                            1,  0,  0;...
                            -1,0,  0;...
                            0,  1,  0;...
                            0,  -1,  0;
                            0,  0,  1;...
                            0,  0,  -1;];
            end
            
            bpid = icub.getBPID('left_arm');
            if bpid > 0
                icub.qlim{bpid} = [
                    deg2rad([-90 10]);
                    deg2rad([0 160]);
                    deg2rad([-37 80]);
                    deg2rad([15 106]);
                    deg2rad([-90 90]);
                    deg2rad([-90 0]);
                    deg2rad([-20 40]);
                    ...
                    deg2rad([0 60]);
                    deg2rad([10 90]);
                    deg2rad([0 90]);
                    deg2rad([0 180]);
                    deg2rad([0 90]);
                    deg2rad([0 180]);
                    deg2rad([0 90]);
                    deg2rad([0 180]);
                    deg2rad([0 270]);
                    ];
                if tightJointLimits
                    icub.qlim{bpid} = [
                        deg2rad([-80 0]);
                        deg2rad([15 120]);
                        deg2rad([-20 60]);
                        deg2rad([15 85]);
                        deg2rad([-30 30]);
                        deg2rad([-50 -5]);
                        deg2rad([-20 20]);
                        ...
                        deg2rad([0 60]);
                        deg2rad([10 90]);
                        deg2rad([0 90]);
                        deg2rad([0 180]);
                        deg2rad([0 90]);
                        deg2rad([0 180]);
                        deg2rad([0 90]);
                        deg2rad([0 180]);
                        deg2rad([0 270]);
                        ];
                end
                icub.qhome{bpid} = deg2rad([-30 30 0 45 0 0 40  15 30 0 0 0 0 0 0 0]);
                %icub.qhome{bpid} = deg2rad([-25 20 15 50 0 0 0  15 30 0 0 0 0 0 0 0]); %sim
                icub.xhome{bpid} = icub.fkineLA([0 0 0], icub.qhome{bpid}, 'rad');
                icub.xlim{bpid} = [-0.33 0.25; -0.11 0.45; -0.16 0.45];%TODO
                icub.jimp{bpid} = [0.2 0; 0.2 0; 0.2 0; 0.2 0; 0.1 0];%soft BEST
                %icub.jimp{bpid} = [0.111 0.014; 0.111 0.014; 0.111 0.014; 0.111 0.014; 0.111 0.014]; %impedance posmode tutorial
                %icub.jimp{bpid} = [0.4 0.03; 0.4 0.03; 0.4 0.03; 0.2 0.01; 0.2 0]; %medium
                icub.hjimp{bpid} = [0.6 0.06; 0.6 0.06; 0.6 0.06; 0.3 0.02; 0.2 0]; %hard
                %icub.jimp{bpid} = [0.4 0.02; 0.4 0.02; 0.4 0.02; 0.2 0.01; 0.2 0]; %own1
                %icub.jimp{bpid} = [0.25 0; 0.25 0; 0.25 0; 0.25 0; 0.1 0]; %own2
            end
            
            
            tightJointLimits = 1;
            
            bpid = icub.getBPID('torso');
            if bpid > 0
                icub.qlim{bpid} = [
                    deg2rad([-40 30]);
                    deg2rad([-70 60]);
                    deg2rad([-55 55]);
                    ];
                if tightJointLimits
                    icub.qlim{bpid} = [
                        deg2rad([-20 20]);
                        deg2rad([-15 15]);
                        deg2rad([-15 15]);
                        ];
                end
                icub.qhome{bpid} = [0 0 0];
                icub.xhome{bpid} = [];
                icub.xlim{bpid} = [];
                icub.jimp{bpid} = [0.1 0; 0.1 0; 0.1 0];
            end
            
            bpid = icub.getBPID('head');
            if bpid > 0
                icub.qlim{bpid} = [
                    deg2rad([-35 22]);
                    deg2rad([-35 35]);
                    deg2rad([-50 50]);
                    deg2rad([-20 10]);
                    deg2rad([-30 30]);
                    deg2rad([0 20]);
                ];
                icub.qhome{bpid} = [0 0 0 0 0 0];
                icub.xhome{bpid} = [];
                icub.xlim{bpid} = [];
                icub.jimp{bpid} = [0.1 0; 0.1 0; 0.1 0; 0.1 0; 0.1 0; 0.1 0];
            end
            
            icub.connected = 0;
        end
        
        function result = checkCableConstraint(icub,q,bpid) %units = 'deg'
            % check is only for sholder joints
            % 4:6 is torso joints are included
            % only arm 1:3
            
            if strcmp(icub.bodyparts{bpid}, 'right_arm') || strcmp(icub.bodyparts{bpid}, 'left_arm')
                [x y] = size(q);
                if x<y
                    q=q';
                end
                
                %result = all(icub.A * q(1:3) +icub.b >0);
                
                 result = all(icub.A * q(1:3) - icub.b- icub.CABLE_WARNING> 0 );
            else
                result =1;
            end
        end
        function setEmotion(icub,i)
            if isnumeric(i)
                yarpWrite(icub.portEmotions, ['set all ' icub.emotions{i}])
                %system(['echo "set all ' icub.emotions{i} '"  | yarp rpc /' icub.robotname '/face/emotions/in'])
            elseif ischar(i)
                yarpWrite(icub.portEmotions, ['set all ' i])
                %system(['echo "set all ' i '"  | yarp rpc /' icub.robotname '/face/emotions/in'])
            else
                disp('[icub] unknown emotion')
            end
        end
        
        function open(icub)
            if icub.connected
                return;
            end
            
            getYarped();
            
            icub.connected = 1;
            thisScope = ['/icubctrlmatlabdbg/' icub.robotname '/'];
            
            if strcmp(icub.robotname, 'icubSim');
                icub.portEmotions  =openPort([thisScope 'emotions:o'] ,'/emotion/in');
            else
                icub.portEmotions  =openPort([thisScope 'emotions:o'] ,['/' icub.robotname '/face/emotions/in']);
                
            end
            for i=1:length(icub.bodyparts)
                icub.options{i} = yarp.Property;
                icub.options{i}.put('robot', icub.robotname);
                icub.options{i}.put('part', icub.bodyparts{i});
                icub.options{i}.put('device', 'remote_controlboard');
                icub.options{i}.put('local', icub.localPorts{i});   % local port names
                icub.options{i}.put('remote', icub.remotePorts{i}); % where we connect to
                
                % Create a device
                icub.devices{i} = yarp.PolyDriver(icub.options{i});
                if ~icub.devices{i}.isValid()
                    warning(['Device for ' icub.bodyparts{i} ' is not valid']);
                    icub.connected = 0;
                end
                icub.pos{i} = icub.devices{i}.viewIPositionControl();
                icub.enc{i} = icub.devices{i}.viewIEncoders();
                icub.mod{i} = icub.devices{i}.viewIControlMode();
                icub.trq{i} = icub.devices{i}.viewITorqueControl();
                icub.imp{i} = icub.devices{i}.viewIImpedanceControl();
                icub.pid{i} = icub.devices{i}.viewIPidControl();
                %icub.vel{i} = icub.devices{i}.viewIVelocityControl();
                %icub.amp{i} = icub.devices{i}.viewIAmplifierControl();
                
                
                thisScope = ['/icubctrlmatlabdbg/' icub.robotname '/'];
                icub.dbglocalPorts{i} = [thisScope filesep icub.bodyparts{i} filesep 'debug'];
                icub.dbgoptions{i} = yarp.Property;
                icub.dbgoptions{i}.put('robot', icub.robotname);
                icub.dbgoptions{i}.put('part', icub.bodyparts{i});
                icub.dbgoptions{i}.put('device', 'debugInterfaceClients');
                icub.dbgoptions{i}.put('local', icub.dbglocalPorts{i});   % local port names
                icub.dbgoptions{i}.put('remote', icub.remotePorts{i}); % where we connect to
                
                icub.dbgdevices{i} = yarp.PolyDriver(icub.dbgoptions{i});
                
                
                if ~icub.dbgdevices{i}.isValid()
                    warning(['Debug Device for ' icub.bodyparts{i} ' is not valid']);
                else
                    icub.dbg{i} = icub.dbgdevices{i}.view();
                end
                
                if strcmp(icub.robotname, 'icub') && (strcmp(icub.bodyparts{i},'left_arm') || strcmp(icub.bodyparts{i},'right_arm'))
                    refSpeed = 40; % 30-40 degree/s for each joint  so be careful!!!
                    icub.setRefSpeed(icub.bodyparts{i}, refSpeed);
                end
            end
            
            if strcmp(icub.robotname, 'icubSim');
                icub.sim.name = 'icubSim';
                icub.sim.world = openPort('simworld', [filesep icub.sim.name filesep 'world']);
            end
        end
        
        function close(icub)
            if ~icub.connected
                return;
            end
            
            icub.positionMode();
            
            for i=1:length(icub.bodyparts)
                icub.devices{i}.close();
            end
            
            if ~isempty(icub.sim)
                icub.sim.world.close();
            end
            if ~isempty(icub.portEmotions)
                icub.portEmotions.close();
            end
            icub.connected = 0;
            disp('disconnected');
        end
        
        function delete(icub)
            icub.close();
            
        end
        
        function positionMode(icub, bodypart)
            if ~icub.connected
                return;
            end
            
            if nargin < 2
                %put everything to position mode
                for i=1:length(icub.bodyparts)
                    for j=1:min(icub.pos{i}.getAxes(), 5)
                        icub.mod{i}.setPositionMode(j-1);
                    end
                end
            else
                bpid = icub.getBPID(bodypart);
                if bpid > 0
                    for j=1:min(icub.pos{bpid}.getAxes(), 5)
                        icub.mod{bpid}.setPositionMode(j-1);
                    end
                end
            end
        end
        
        function ztrqMode(icub, bodypart)
            if ~icub.connected
                return;
            end
            
            if nargin < 2
                %put everything to zero torque mode
                for i=1:length(icub.bodyparts)
                    for j=1:min(icub.pos{i}.getAxes(), 5)
                        icub.imp{i}.setImpedance(j-1, 0, 0);
                    end
                    for j=1:min(icub.pos{i}.getAxes(), 5)
                        icub.mod{i}.setTorqueMode(j-1);
                    end
                end
            else
                bpid = icub.getBPID(bodypart);
                if bpid > 0
                    for j=1:min(icub.pos{bpid}.getAxes(), 5)
                        icub.imp{bpid}.setImpedance(j-1, 0, 0);
                    end
                    for j=1:min(icub.pos{bpid}.getAxes(), 5)
                        icub.mod{bpid}.setTorqueMode(j-1);
                    end
                end
            end
        end
        
        function jimpMode(icub, bodypart)
            if ~icub.connected
                return;
            end
            
            if nargin < 2
                %put everything to joint impedance mode
                for i=1:length(icub.bodyparts)
                    for j=1:min(icub.pos{i}.getAxes(), 5)
                        icub.imp{i}.setImpedance(j-1, icub.jimp{i}(j,1), icub.jimp{i}(j,2));
                    end
                    for j=1:min(icub.pos{i}.getAxes(), 5)
                        icub.mod{i}.setImpedancePositionMode(j-1);
                    end
                end
            else
                bpid = icub.getBPID(bodypart);
                if bpid > 0
                    for j=1:min(icub.pos{bpid}.getAxes(), 5)
                        icub.imp{bpid}.setImpedance(j-1, icub.jimp{bpid}(j,1), icub.jimp{bpid}(j,2));
                    end
                    for j=1:min(icub.pos{bpid}.getAxes(), 5)
                        icub.mod{bpid}.setImpedancePositionMode(j-1);
                    end
                end
            end
        end
        
        function hjimpMode(icub, bodypart)
            if ~icub.connected
                return;
            end
            
            if nargin < 2
                %put everything to joint impedance mode
                for i=1:length(icub.bodyparts)
                    for j=1:min(icub.pos{i}.getAxes(), 5)
                        icub.imp{i}.setImpedance(j-1, icub.hjimp{i}(j,1), icub.hjimp{i}(j,2));
                    end
                    for j=1:min(icub.pos{i}.getAxes(), 5)
                        icub.mod{i}.setImpedancePositionMode(j-1);
                    end
                end
            else
                bpid = icub.getBPID(bodypart);
                if bpid > 0
                    for j=1:min(icub.pos{bpid}.getAxes(), 5)
                        icub.imp{bpid}.setImpedance(j-1, icub.hjimp{bpid}(j,1), icub.hjimp{bpid}(j,2));
                    end
                    for j=1:min(icub.pos{bpid}.getAxes(), 5)
                        icub.mod{bpid}.setImpedancePositionMode(j-1);
                    end
                end
            end
        end
        
        function setFirmwarePidGains(icub, bodypart, kp, ki, kd, shift, maxout, fc)
            if ~icub.connected
                return;
            end
            
            bpid = icub.getBPID(bodypart);
            if bpid <=0
                return
            end
            
            numJoints = icub.enc{bpid}.getAxes();
            
            pids = cell(numJoints,1);
            for i=1:numJoints
                pids{i} = yarp.Pid;
            end
            
            %trq{bpid}.getTorquePids(pids);
            
            d = 1;
            if bpid == icub.getBPID('left_arm')
                d = -1;
            end
            
            for i=1:5
                pids{i}.kp = d*kp(i);
                pids{i}.kd = d*kd(i);
                pids{i}.ki = d*ki(i);
                pids{i}.scale = shift(i);
                pids{i}.max_output = maxout(i);
                if ~icub.dbg{bpid}.setDebugParameter(i-1, 5,  icub.filt(fc(i)))
                    disp(['Error while setting fc of joint ' num2str(i)]);
                end
            end
            
            for i=1:numJoints
                icub.trq{bpid}.setTorquePid(i-1, pids{i});
            end
        end
        
        function q = getPosture(icub, bodypart, units)
            if ~icub.connected
                return;
            end
            
            if nargin < 3
                units = 'rad';
            end
            
            bpid = icub.getBPID(bodypart);
            
            q = zeros(1,icub.enc{bpid}.getAxes());
            for i=1:icub.enc{bpid}.getAxes()
                q(i) = icub.enc{bpid}.getEncoder(i-1);
            end
            
            if strcmp(units, 'rad')
                q = deg2rad(q);
            elseif strcmp(units, 'norm')
                q = deg2rad(q);
                q = icub.norm2range(bpid, q);
            end
        end
        
        function setRefSpeed(icub, bodypart, rf)
            if ~icub.connected
                return;
            end
            
            bpid = icub.getBPID(bodypart);
            if bpid < 0
                return;
            end
            qrf = yarp.DVector(icub.pos{bpid}.getAxes());
            for i=1:min(icub.pos{bpid}.getAxes())
                qrf.set(i-1, rf);
            end
            icub.pos{bpid}.setRefSpeeds(qrf);
        end
        
        function driveToPosture(icub, bodypart, q, blocking, units)
            if ~icub.connected
                return;
            end
            
            if nargin < 4
                blocking = 0;
            end
            if nargin < 5
                units = 'rad';
            end
            
            bpid = icub.getBPID(bodypart);
            if bpid < 0
                return;
            end
            
            %format and clip joint angle commands
            if strcmp(units, 'rad')
                q = min(max(q', icub.qlim{bpid}(1:length(q),1)), icub.qlim{bpid}(1:length(q),2))';
                q = rad2deg(q);
            elseif strcmp(units, 'deg')
                q = min(max(q', rad2deg(icub.qlim{bpid}(1:length(q),1))), rad2deg(icub.qlim{bpid}(1:length(q),2)))';
            elseif strcmp(units, 'norm')
                q = icub.norm2range(bodypart, q);
                q = min(max(q', icub.qlim{bpid}(1:length(q),1)), icub.qlim{bpid}(1:length(q),2))';
                q = rad2deg(q);
            end
            
            if blocking
                qNew = zeros(1,icub.enc{bpid}.getAxes());
            end
            if icub.checkCableConstraint(q,bpid);
                
                qy = yarp.DVector(icub.pos{bpid}.getAxes());
                for i=1:min(icub.pos{bpid}.getAxes(),length(q))
                    qy.set(i-1, q(i));
                end
                if strcmp(icub.bodyparts{bpid}, 'right_arm') || strcmp(icub.bodyparts{bpid}, 'left_arm')
                    %keep uncontrolled parts at current position
                    for i=min(icub.pos{bpid}.getAxes(),length(q))+1:icub.pos{bpid}.getAxes()
                        qy.set(i-1, icub.enc{bpid}.getEncoder(i-1));
                    end
                end
                
                if blocking
                    minSteps = 30;
                    maxSteps = 1000;
                    maxDq = 1e-4;
                    cnt = 0;
                    
                    qOld = 99999.9 * ones(1, icub.enc{bpid}.getAxes());
                    dq = 99999.9;
                    
                    icub.pos{bpid}.positionMove(qy);
                    
                    while (cnt < minSteps) || ((cnt < maxSteps) && (dq > maxDq))
                        %icub.pos{bpid}.positionMove(qy);
                        for i=1:length(qNew)
                            qNew(i) = icub.enc{bpid}.getEncoder(i-1);
                        end
                        dq = sum((qNew - qOld).^2);
                        qOld = qNew;
                        
                        pause(0.1);
                        cnt = cnt + 1;
                    end
                    
                    fprintf(['driveToPosture for ' icub.bodyparts{bpid} ' in ' num2str(cnt) ' steps\n']);
                else
                    icub.pos{bpid}.positionMove(qy);
                end
            else
                warning(['[' icub.bodyparts{bpid} '] cable is not long enough to reach position']);
            end
        end
        
        function driveToPostureBodyBlocking(icub, qt, qla, qra, units)
            if ~icub.connected
                return;
            end
            if nargin < 5
                units = 'rad';
            end
            
            bpidtorso = icub.getBPID('torso');
            bpidla = icub.getBPID('left_arm');
            bpidra = icub.getBPID('right_arm');
            
            %format and clip joint angle commands
            if strcmp(units, 'rad')
                qt = min(max(qt', icub.qlim{bpidtorso}(1:length(qt),1)), icub.qlim{bpidtorso}(1:length(qt),2))';
                qt = rad2deg(qt);
                qla = min(max(qla', icub.qlim{bpidla}(1:length(qla),1)), icub.qlim{bpidla}(1:length(qla),2))';
                qla = rad2deg(qla);
                qra = min(max(qra', icub.qlim{bpidra}(1:length(qra),1)), icub.qlim{bpidra}(1:length(qra),2))';
                qra = rad2deg(qra);
            elseif strcmp(units, 'deg')
                qt = min(max(qt', rad2deg(icub.qlim{bpidtorso}(1:length(qt),1))), rad2deg(icub.qlim{bpidtorso}(1:length(qt),2)))';
                qla = min(max(qla', rad2deg(icub.qlim{bpidla}(1:length(qla),1))), rad2deg(icub.qlim{bpidla}(1:length(qla),2)))';
                qra = min(max(qra', rad2deg(icub.qlim{bpidra}(1:length(qra),1))), rad2deg(icub.qlim{bpidra}(1:length(qra),2)))';
            elseif strcmp(units, 'norm')
                qt = icub.norm2range(bpidtorso, qt);
                qt = min(max(qt', icub.qlim{bpidtorso}(1:length(qt),1)), icub.qlim{bpidtorso}(1:length(qt),2))';
                qt = rad2deg(qt);
                qla = icub.norm2range(bpidla, qla);
                qla = min(max(qla', icub.qlim{bpidla}(1:length(qla),1)), icub.qlim{bpidla}(1:length(qla),2))';
                qla = rad2deg(qla);
                qra = icub.norm2range(bpidra, qra);
                qra = min(max(qra', icub.qlim{bpidra}(1:length(qra),1)), icub.qlim{bpidra}(1:length(qra),2))';
                qra = rad2deg(qra);
            end
            
            qty = yarp.DVector(icub.pos{bpidtorso}.getAxes());
            for i=1:min(icub.pos{bpidtorso}.getAxes(),length(qt))
                qty.set(i-1, qt(i));
            end
            qlay = yarp.DVector(icub.pos{bpidla}.getAxes());
            for i=1:min(icub.pos{bpidla}.getAxes(),length(qla))
                qlay.set(i-1, qla(i));
            end
            qray = yarp.DVector(icub.pos{bpidra}.getAxes());
            for i=1:min(icub.pos{bpidra}.getAxes(),length(qra))
                qray.set(i-1, qra(i));
            end
            %keep uncontrolled parts at current position
            for i=min(icub.pos{bpidla}.getAxes(),length(qla))+1:icub.pos{bpidla}.getAxes()
                qlay.set(i-1, icub.enc{bpidla}.getEncoder(i-1));
            end
            for i=min(icub.pos{bpidra}.getAxes(),length(qra))+1:icub.pos{bpidra}.getAxes()
                qray.set(i-1, icub.enc{bpidra}.getEncoder(i-1));
            end
            
            minSteps = 10;
            maxSteps = 300;
            maxDq = 1e-1;
            cnt = 0;
            
            qNew = zeros(1,icub.enc{bpidtorso}.getAxes() + icub.enc{bpidla}.getAxes() + icub.enc{bpidra}.getAxes());
            qOld = 99999.9 * ones(size(qNew));
            dq = 99999.9;
            
            icub.pos{bpidtorso}.positionMove(qty);
            icub.pos{bpidla}.positionMove(qlay);
            icub.pos{bpidra}.positionMove(qray);
            
            while (cnt < minSteps) || ((cnt < maxSteps) && (dq > maxDq))
                %icub.pos{bpidtorso}.positionMove(qty);
                %icub.pos{bpidla}.positionMove(qlay);
                %icub.pos{bpidra}.positionMove(qray);
                for i=1:qty.size
                    qNew(i) = icub.enc{bpidtorso}.getEncoder(i-1);
                end
                for i=1:qlay.size
                    qNew(i+qty.size) = icub.enc{bpidla}.getEncoder(i-1);
                end
                for i=1:qray.size
                    qNew(i+qty.size+qlay.size) = icub.enc{bpidra}.getEncoder(i-1);
                end
                dq = sum((qNew - qOld).^2);
                qOld = qNew;
                
                pause(0.05);
                cnt = cnt + 1;
            end
            fprintf(['driveToPostureBodyBlocking converged in ' num2str(cnt) ' steps\n']);
        end
        
        function driveHand(icub, bodypart, q, units)
            if ~icub.connected
                return;
            end
            
            if nargin < 4
                units = 'rad';
            end
            
            bpid = icub.getBPID(bodypart);
            if bpid < 0
                return;
            end
            
            if strcmp(icub.bodyparts{bpid}, 'right_arm') == 0 && strcmp(icub.bodyparts{bpid}, 'left_arm') == 0
                return;
            end
            
            %format and clip joint angle commands
            if strcmp(units, 'rad')
                q = min(max(q', icub.qlim{bpid}(8:end,1)), icub.qlim{bpid}(8:end,2))';
                q = rad2deg(q);
            elseif strcmp(units, 'deg')
                q = min(max(q', rad2deg(icub.qlim{bpid}(8:end,1))), rad2deg(icub.qlim{bpid}(8:end,2)))';
            elseif strcmp(units, 'norm')
                return;
                %q = icub.norm2range(bodypart, q);
                %q = min(max(q', icub.qlim{bpid}(8:end,1)), icub.qlim{bpid}(8:end,2))';
                %q = rad2deg(q);
            end
            
            qy = yarp.DVector(icub.pos{bpid}.getAxes());
            %keep uncontrolled parts at current position
            for i=1:icub.pos{bpid}.getAxes()
                qy.set(i-1, icub.enc{bpid}.getEncoder(i-1));
            end
            for i=8:icub.pos{bpid}.getAxes()
                qy.set(i-1, q(i-7));
            end
            
            icub.pos{bpid}.positionMove(qy);
        end
        
        function bpid = getBPID(icub, bodypart)
            bpid = -1;
            if isnumeric(bodypart)
                bpid = bodypart;
            elseif ischar(bodypart)
                for i=1:length(icub.bodyparts)
                    if strcmp(bodypart, icub.bodyparts{i})
                        bpid = i;
                        break;
                    end
                end
            end
            if bpid < 0
                warning('bodypart not found/initialized');
            end
        end
        
        function setSimView(icub,view)
            if ~icub.connected || isempty(icub.sim)
                return;
            end
            
            if ischar(view)
                if strcmp(view, 'frontal')
                    request(icub.sim.world, ['view ' icub.views{1}]);
                elseif strcmp(view, 'top')
                    request(icub.sim.world, ['view ' icub.views{2}]);
                elseif strcmp(view, 'right')
                    request(icub.sim.world, ['view ' icub.views{6}]);
                elseif strcmp(view, 'left')
                    request(icub.sim.world, ['view ' icub.views{7}]);
                elseif strcmp(view, 'side-right')
                    request(icub.sim.world, ['view ' icub.views{8}]);
                elseif strcmp(view, 'side-left')
                    request(icub.sim.world, ['view ' icub.views{9}]);
                elseif strcmp(view, 'top-left')
                    request(icub.sim.world, ['view ' icub.views{5}]);
                elseif strcmp(view, 'top-right')
                    request(icub.sim.world, ['view ' icub.views{4}]);
                elseif strcmp(view, 'top-front')
                    request(icub.sim.world, ['view ' icub.views{3}]);
                else
                    warning('no view');
                end
            else
                if isrow(view) && length(view) == 5
                    request(icub.sim.world, ['view ' num2str(view)]);
                else
                    warning('invalid view request');
                end
            end
        end
        
        function capture(icub, imgname, relativePath)
            if ~icub.connected || isempty(icub.sim)
                return;
            end
            if nargin < 2
                imgname = 'img';
            end
            if nargin < 3
                relativePath = '.';
            end
            
            request(icub.sim.world, ['saveview ' relativePath filesep imgname '.ppm']);
        end
        
        function [X, R] = fkineRA(icub, Qtorso, Qarm, units)
            if nargin < 4
                units = 'rad';
            end
            
            if strcmp(units, 'norm')
                Qtorso = icub.norm2rad(icub.getBPID('torso'), Qtorso);
                Qarm = icub.norm2rad(icub.getBPID('right_arm'), Qarm);
            elseif strcmp(units, 'deg')
                Qtorso = deg2rad(Qtorso);
                Qarm = deg2rad(Qarm);
            end
            
            if size(Qarm,2) < 7
                Qarm = [Qarm zeros(size(Qarm,1), 7-size(Qarm,2))];
            end
            
            X = zeros(size(Qtorso, 1), 3);
            if nargout > 1
                R = zeros(size(Qtorso, 1), 3, 3);
            end
            
            numJointsArm = size(Qarm, 2);
            pad = zeros(1, max(0, 7 - numJointsArm));
            %TODO do not pad zeros but home posture values!!!
            
            for i=1:size(Qtorso, 1)
                wtheta0 = Qtorso(i,3);
                wtheta1 = Qtorso(i,2);
                wtheta2 = Qtorso(i,1);
                
                %Reference frames attached to links
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %               Waist                       *
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                G_01=evalDHMatrix(      .032,       0,     pi/2,                wtheta0);
                G_12=evalDHMatrix(       0,    -.0055,     pi/2,                wtheta1-pi/2);
                G_23=evalDHMatrix(-.0233647,  -.1433,     pi/2,                wtheta2 - 15*pi/180 - pi/2 );
                
                G_sL0 = [rotz(-pi/2) zeros(3,1); zeros(1,3) 1]*[0 0 1 0; 0 -1 0 0; 1 0 0 0; 0 0 0 1];
                G_sL1 = G_sL0*G_01;
                G_sL2 = G_sL1*G_12;
                G_sL3 = G_sL2*G_23;
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %               LeftArm                     *
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                G_34=evalDHMatrix(    0, -.10774,     pi/2,    Qarm(i,1)-pi/2);
                G_45=evalDHMatrix(    0,       0,    -pi/2,    Qarm(i,2)-pi/2);
                G_56=evalDHMatrix(  -.015, -.15228,    -pi/2,    Qarm(i,3)-pi/2-15*pi/180);
                G_67=evalDHMatrix(   .015,       0,     pi/2,    Qarm(i,4));
                G_78=evalDHMatrix(    0,  -.1373,     pi/2,    Qarm(i,5)-pi/2);
                G_89=evalDHMatrix(    0,       0,     pi/2,    Qarm(i,6)+pi/2);
                G_910=evalDHMatrix( .0625,     .016,        0,    Qarm(i,7)+pi);
                
                %Reference frames attached to links
                G_sL4   = G_sL3  * G_34;
                G_sL5   = G_sL4  * G_45;
                G_sL6   = G_sL5  * G_56;
                G_sL7   = G_sL6  * G_67;
                G_sL8   = G_sL7  * G_78;
                G_sL9   = G_sL8  * G_89;
                G_sL10  = G_sL9  * G_910;
                XE=[0 0 0 1]';
                %x = G_sL10*XE;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                
                T_Ro0 = G_sL0;
                T_0n  = G_01*G_12*G_23*G_34*G_45*G_56*G_67*G_78*G_89*G_910;
                
                xee = [T_0n(1,4), T_0n(2,4), T_0n(3,4)]';
                %xee = xee + [0; 0.015; 0.015]; %in palm
                xeer = T_Ro0(1:3,1:3) * xee;
                X(i,:) = xeer';
                R = T_Ro0(1:3,1:3) *T_0n(1:3,1:3);
            end
        end
        
        function [X, R] = fkineLA(icub, Qtorso, Qarm, units)
            if nargin < 4
                units = 'rad';
            end
            
            if strcmp(units, 'norm')
                Qtorso = icub.norm2rad(icub.getBPID('torso'), Qtorso);
                Qarm = icub.norm2rad(icub.getBPID('left_arm'), Qarm);
            elseif strcmp(units, 'deg')
                Qtorso = deg2rad(Qtorso);
                Qarm = deg2rad(Qarm);
            end
            
            if size(Qarm,2) < 7
                Qarm = [Qarm zeros(size(Qarm,1), 7-size(Qarm,2))];
            end
            
            X = zeros(size(Qtorso, 1), 3);
            if nargout > 1
                R = zeros(size(Qtorso, 1), 3, 3);
            end
            
            numJointsArm = size(Qarm, 2);
            pad = zeros(1, max(0, 7 - numJointsArm));
            %TODO do not pad zeros but home posture values!!!
            
            for i=1:size(Qtorso, 1)
                wtheta0 = Qtorso(i,3);
                wtheta1 = Qtorso(i,2);
                wtheta2 = Qtorso(i,1);
                
                %Reference frames attached to links
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %               Waist                       *
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                G_01=evalDHMatrix(     .032,       0,     pi/2,                wtheta0);
                G_12=evalDHMatrix(      0,    -.0055,     pi/2,                wtheta1-pi/2);
                G_23=evalDHMatrix(.0233647,  -.1433,    -pi/2,                wtheta2 + 15*pi/180 + pi/2 );
                
                G_sL0 = [rotz(-pi/2) zeros(3,1); zeros(1,3) 1]*[0 0 1 0; 0 -1 0 0; 1 0 0 0; 0 0 0 1];
                G_sL1 = G_sL0*G_01;
                G_sL2 = G_sL1*G_12;
                G_sL3 = G_sL2*G_23;
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %               LeftArm                     *
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                G_34=evalDHMatrix(   0,  .10774,    -pi/2, Qarm(i,1)+pi/2);
                G_45=evalDHMatrix(   0,       0,     pi/2, Qarm(i,2)-pi/2);
                G_56=evalDHMatrix(  .015,  .15228,    -pi/2, Qarm(i,3)+pi/2-15*pi/180);
                G_67=evalDHMatrix( -.015,      0,     pi/2,  Qarm(i,4));
                G_78=evalDHMatrix(   0,   .1373,     pi/2, Qarm(i,5)-pi/2);
                G_89=evalDHMatrix(   0,       0,     pi/2, Qarm(i,6)+pi/2);
                G_910=evalDHMatrix(.0625,    -.016,        0, Qarm(i,7));
                
                %Reference frames attached to links
                G_sL4   = G_sL3  * G_34;
                G_sL5   = G_sL4  * G_45;
                G_sL6   = G_sL5  * G_56;
                G_sL7   = G_sL6  * G_67;
                G_sL8   = G_sL7  * G_78;
                G_sL9   = G_sL8  * G_89;
                G_sL10  = G_sL9  * G_910;
                XE=[0 0 0 1]';
                %x = G_sL10*XE;
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                
                T_Ro0 = G_sL0;
                T_0n  = G_01*G_12*G_23*G_34*G_45*G_56*G_67*G_78*G_89*G_910;
                
                xee = [T_0n(1,4), T_0n(2,4), T_0n(3,4)]';
                %xee = xee + [0; 0.015; 0.015]; %in palm
                xeer = T_Ro0(1:3,1:3) * xee;
                X(i,:) = xeer';
                R = T_Ro0(1:3,1:3)* T_0n(1:3,1:3);
            end
        end
        
        
        
        function Qnorm = rad2norm(icub, bodypart, Q)
            bpid = icub.getBPID(bodypart);
            Qnorm = range2norm(Q, icub.qlim{bpid}(1:size(Q,2),:)');
        end
        
        function Q = norm2rad(icub, bodypart, Qnorm)
            bpid = icub.getBPID(bodypart);
            Q = norm2range(Qnorm, icub.qlim{bpid}(1:size(Qnorm,2),:)');
        end
        
        function Xnorm = range2norm(icub, bodypart, X)
            bpid = icub.getBPID(bodypart);
            Xnorm = range2norm(X, icub.xlim{bpid});
        end
        
        function X = norm2range(icub, bodypart, Xnorm)
            bpid = icub.getBPID(bodypart);
            X = norm2range(Xnorm, icub.xlim{bpid});
        end
        
        %utility for pid gains
        function v = filt(icub, v)
            sampling = 1000;
            v = v/1000; %from mHz to Hz
            s = -2*tan(pi*v/sampling); %v frequency in hertz
            v = (2+s)/(2-s);
            v = v * 10000;
        end
        
        
        
        function addPaddle(icub)
            if ~icub.connected || isempty(icub.sim)
                return;
            end
            
            request(icub.sim.world, 'world del all');
            request(icub.sim.world, 'world mk scyl 0.01 0.9 300 1 1 1 0 0');
            %request(icub.sim.world, 'world mk scyl 0.02 1 300 1 1 1 0 0'); %real
        end
        
        function drawPaddle(icub)
            if ~icub.connected || isempty(icub.sim)
                return;
            end
            
            qt = icub.getPosture('torso');
            ql = icub.getPosture('left_arm');
            qr = icub.getPosture('right_arm');
            
            xl = icub.fkineLA(qt, ql);
            xr = icub.fkineRA(qt, qr);
            xlr = xr - xl;
            xl = xl + 0.5 * xlr;
            
            xs = [-xl(2) xl(3) -xl(1)] + [200 0.5976 -0.026];
            request(icub.sim.world, ['world set scyl 1 ' num2str(xs)]);
            
            xlr = [-xlr(2) xlr(3) -xlr(1)];
            R = generateMatrix(xlr, 0);
            [r1 r2 r3] = dcm2angles(R, 'XYZ');
            e = rad2deg([r1 r2 r3]);
            request(icub.sim.world, ['world rot scyl 1 ' num2str(e)]);
        end
        
        function tightGrasp(icub, bodyPart)
            icub.driveHand(bodyPart, [5 70 30 65 50 70 60 75 100], 'deg');
        end
        
        function looseGrasp(icub, bodyPart)
            icub.driveHand(bodyPart, [5 70 25 50 45 55 55 65 75], 'deg');
        end
        
        function releaseGrasp(icub, bodyPart)
            icub.driveHand(bodyPart, [0 10 5 4 5 3 7 0 9], 'deg');
        end
        
    end%methods
    
end%class

