classdef ICubUISimRob < handle
    %ICUBUI Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        icub = {[], []};
        icubexe = {1, 0};
        icubnames = {'icubSim', 'icub'};
        bodyparts = {'left_arm', 'right_arm', 'head'};
        
        app = [];
        f = [];
        parentup = [];
        uis = [];
        
        SIMIDX = 1;
        ROBIDX = 2;
    end
    
    methods
        function ui = ICubUISimRob(app, parentup)
            ui.app = app;
            if nargin < 2
                ui.f = figure('Visible','off','Position',[400,300,800,700]);
                set(ui.f,'MenuBar','none');
                set(ui.f,'Units','normalized');
                set(ui.f,'Name','ICubUISimRob');
                movegui(ui.f,'center');
                set(ui.f,'Visible','on');
                ui.parentup = ui.f;
            else
                ui.parentup = parentup;
            end

            ui.initUI();
        end
        
        function connect(ui,src,evn)
            idx = -1;
            if src == ui.uis.connectSim
                idx = ui.SIMIDX;
            end
            if src == ui.uis.connectRob
                idx = ui.ROBIDX;
            end
            if idx <= 0 || idx > 2
                return;
            end
                
            if ~isempty(ui.icub{idx})
                ui.icub{idx}.close();
                ui.icub{idx} = [];
            end
            
            rob = ui.icubnames{idx}
            ui.icub{idx} = ICub(ui.icubnames{idx}, ui.bodyparts);
            ui.icub{idx}.open();
            
            if ui.icub{idx}.connected
                set(src, 'Enable', 'off');
            else
                ui.icub{idx}.close();
                ui.icub{idx} = [];
            end
            
            isconnected = 0;
            for i=1:2
                if ~isempty(ui.icub{i})
                    isconnected = 1;
                end
            end
            %if isconnected
                ui.app.stateUI.setState(ui.app.WAITFORSTART);
            %else
            %    ui.app.stateUI.setState(ui.app.PRECONNECT);
            %end
        end
        
        function disconnect(ui,src,evn)
            idx = -1;
            if src == ui.uis.disconnectSim
                idx = ui.SIMIDX;
            end
            if src == ui.uis.disconnectRob
                idx = ui.ROBIDX;
            end
            if idx <= 0 || idx > 2
                return;
            end
            
            if ~isempty(ui.icub{idx})
                if ui.icub{idx}.connected && ~isempty(ui.icub{idx}.sim)
                    yarpWrite(ui.icub{idx}.sim.world, 'world del all');
                    ui.icub{idx}.close();
                    disp('disconnected simulation');
                else
                    ui.icub{idx}.close();
                    disp('disconnected from robot');
                end
            end
          
            ui.icub{idx} = [];

            if idx == ui.SIMIDX
                set(ui.uis.connectSim, 'Enable', 'on');
            end
            if idx == ui.ROBIDX
                set(ui.uis.connectRob, 'Enable', 'on');
            end
            
            %if ui.isconnected() || ui.app.
                ui.app.stateUI.setState(ui.app.WAITFORSTART);
            %else
            %    ui.app.stateUI.setState(ui.app.PRECONNECT);
            %end
        end
        
        function b = isconnected(ui)
            b = 0;
            for i=1:2
                if ~isempty(ui.icub{i})
                    b = 1;
                end
            end
        end
        
        function execute(ui,src,evn)
            idx = -1;
            if src == ui.uis.exeSim
                idx = ui.SIMIDX;
            end
            if src == ui.uis.exeRob
                idx = ui.ROBIDX;
            end
            if idx <= 0 || idx > 2
                return;
            end
            
            ui.icubexe{idx} = get(src, 'Value');
        end
        
        function setEmotion(ui,e)
            for i=1:2
                if ui.icubexe{i}
                    if ~isempty(ui.icub{i})
                        ui.icub{i}.setEmotion(e);
                    end
                end
            end
        end
        
        function positionMode(ui, bodypart)
            if nargin < 2
                for i=1:2
                    if ui.icubexe{i}
                        if ~isempty(ui.icub{i})
                            ui.icub{i}.positionMode();
                        end
                    end
                end
            else
                for i=1:2
                    if ui.icubexe{i}
                        if ~isempty(ui.icub{i})
                            ui.icub{i}.positionMode(bodypart);
                        end
                    end
                end
            end
        end
        
        function jimpMode(ui, bodypart)
            for i=1:2
                if ui.icubexe{i}
                    if ~isempty(ui.icub{i})
                        ui.icub{i}.jimpMode(bodypart);
                    end
                end
            end
        end
        
        function hjimpMode(ui, bodypart)
            for i=1:2
                if ui.icubexe{i}
                    if ~isempty(ui.icub{i})
                        ui.icub{i}.hjimpMode(bodypart);
                    end
                end
            end
        end
        
        function q = getPosture(ui, bodypart, units)
            if nargin < 3
                units = 'rad';
            end
            
            if ui.icubexe{ui.ROBIDX}
                if ~isempty(ui.icub{ui.ROBIDX})
                    q = ui.icub{ui.ROBIDX}.getPosture(bodypart, units);
                    return;
                end
            end
            if ui.icubexe{ui.SIMIDX}
                if ~isempty(ui.icub{ui.SIMIDX})
                    q = ui.icub{ui.SIMIDX}.getPosture(bodypart, units);
                    return;
                end
            end
            warning('no robot/simulation selected or available');
            q = [];
        end
        
        function driveToPosture(ui, bodypart, q, blocking, units)
            if nargin < 4
                blocking = 0;
            end
            if nargin < 5
                units = 'rad';
            end
            
            for i=1:2
                if ui.icubexe{i}
                    if ~isempty(ui.icub{i})
                        ui.icub{i}.driveToPosture(bodypart, q, blocking, units);
                    end
                end
            end
        end
        
        function driveToPostureBodyBlocking(ui, qla, qra, units)
            if nargin < 5
                units = 'rad';
            end
            
            if ui.icubexe{ui.ROBIDX}
                if ~isempty(ui.icub{ui.ROBIDX})
                    %non-blocking call on simulator
                    if ui.icubexe{ui.SIMIDX}
                        if ~isempty(ui.icub{ui.SIMIDX})
                            ui.icub{ui.SIMIDX}.driveToPosture('left_arm', qla);
                            ui.icub{ui.SIMIDX}.driveToPosture('right_arm', qra);
                        end
                    end
                    
                    %blocking call on real robot
                    ui.icub{ui.ROBIDX}.driveToPostureBodyBlocking(qla, qra, units);
                    return;
                end
            end
            
            %blocking call on simulator
            if ui.icubexe{ui.SIMIDX}
                if ~isempty(ui.icub{ui.SIMIDX})
                    ui.icub{ui.SIMIDX}.driveToPostureBodyBlocking(qla, qra, units);
                end
            end
        end
        
        function driveWithVelocity(ui, bodypart, dq)         
            for i=1:2
                if ui.icubexe{i}
                    if ~isempty(ui.icub{i})
                        ui.icub{i}.driveWithVelocity(bodypart, dq);
                    end
                end
            end
        end
        
        function driveHand(ui, bodypart, q, units)
            if nargin < 4
                units = 'rad';
            end
            
            for i=1:2
                if ui.icubexe{i}
                    if ~isempty(ui.icub{i})
                        ui.icub{i}.driveHand(bodypart, q, units);
                    end
                end
            end
        end
        
        function addPaddle(ui)
            if ui.icubexe{ui.SIMIDX}
                if ~isempty(ui.icub{ui.SIMIDX})
                    ui.icub{ui.SIMIDX}.addPaddle();
                end
            end
        end
        
        function drawPaddle(ui)
            if ui.icubexe{ui.SIMIDX}
                if ~isempty(ui.icub{ui.SIMIDX})
                    ui.icub{ui.SIMIDX}.drawPaddle();
                end
            end
        end
        
        function delete(ui)
            %ui.disconnect(ui.uis.disconnectSim);
            %ui.disconnect(ui.uis.disconnectRob);
            for i=1:2
                if ~isempty(ui.icub{i})
                    delete(ui.icub{i});
                end
            end
        end
        
        function view(ui,src,evn)
            if ~isempty(ui.icub{ui.SIMIDX})
                ui.icub{ui.SIMIDX}.setSimView(get(src,'String'));
            end
        end
        
        function capture(ui,src,evn)
            if ~isempty(ui.icub{ui.SIMIDX})
                str = get(ui.uis.captureName,'String');
                num = round(str2num(str));
                numstr = sprintf('%03d', num);
                ui.icub{ui.SIMIDX}.capture(['icub-' numstr]);
                
                set(ui.uis.captureName,'String', sprintf('%03d', num+1));
            end
        end
        
        function initUI(ui)
            %%%%%%%%% iCubYarp %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            ui.uis.iCubSimUIPanel = uipanel('Title', 'iCubSim',...
                'Parent',ui.parentup,...
                'Units','normalized',...
                'Position', [0 1-0.5 0.5 0.5],...
                'FontSize', 12, 'BackgroundColor', 'white');
            ui.uis.iCubRobUIPanel = uipanel('Title', 'iCubRob',...
                'Parent',ui.parentup,...
                'Units','normalized',...
                'Position', [0.5 1-0.5 0.5 0.5],...
                'FontSize', 12, 'BackgroundColor', 'white');
            ui.uis.connectSim = uicontrol('Style', 'pushbutton',...
                'String','connect',...
                'Parent',ui.uis.iCubSimUIPanel,'Units','normalized','Position', [0 0.66 1 0.33],...
                'FontSize', 12,'Callback',@ui.connect);
            ui.uis.connectRob = uicontrol('Style', 'pushbutton',...
                'String','connect',...
                'Parent',ui.uis.iCubRobUIPanel,'Units','normalized','Position', [0 0.66 1 0.33],...
                'FontSize', 12,'Callback',@ui.connect);
            ui.uis.disconnectSim = uicontrol('Style', 'pushbutton',...
                'String','disconnect',...
                'Parent',ui.uis.iCubSimUIPanel,'Units','normalized','Position', [0 0.33 1 0.33],...
                'FontSize', 12,'Callback',@ui.disconnect);
            ui.uis.disconnectRob = uicontrol('Style', 'pushbutton',...
                'String','disconnect',...
                'Parent',ui.uis.iCubRobUIPanel,'Units','normalized','Position', [0 0.33 1 0.33],...
                'FontSize', 12,'Callback',@ui.disconnect);
            ui.uis.exeSim = uicontrol('Style', 'togglebutton',...
                'String','execute','Min',0,'Max',1,'Value',ui.icubexe{ui.SIMIDX},...
                'Parent',ui.uis.iCubSimUIPanel,'Units','normalized','Position', [0 0 1 0.33],...
                'FontSize', 12,'Callback',@ui.execute);
            ui.uis.exeRob = uicontrol('Style', 'togglebutton',...
                'String','execute','Min',0,'Max',1,'Value',ui.icubexe{ui.ROBIDX},...
                'Parent',ui.uis.iCubRobUIPanel,'Units','normalized','Position', [0 0 1 0.33],...
                'FontSize', 12,'Callback',@ui.execute);
            
            %%%%%%%%% View Control %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            w = 0.2;
            ui.uis.iCubViewCtrlUIPanel = uipanel('Title', 'iCubSim View',...
                'Parent',ui.parentup,...
                'Units','normalized',...
                'Position', [0 1-0.8 1 0.3],...
                'FontSize', 12, 'BackgroundColor', 'white');
            ui.uis.viewFrontal = uicontrol('Style', 'pushbutton',...
                'String','frontal',...
                'Parent',ui.uis.iCubViewCtrlUIPanel,'Units','normalized','Position', [0 0.5 w 0.5],...
                'FontSize', 12,'Callback',@ui.view);
            ui.uis.viewTop = uicontrol('Style', 'pushbutton',...
                'String','top',...
                'Parent',ui.uis.iCubViewCtrlUIPanel,'Units','normalized','Position', [0 0 w 0.5],...
                'FontSize', 12,'Callback',@ui.view);
            ui.uis.viewRight = uicontrol('Style', 'pushbutton',...
                'String','right',...
                'Parent',ui.uis.iCubViewCtrlUIPanel,'Units','normalized','Position', [w 0.5 w 0.5],...
                'FontSize', 12,'Callback',@ui.view);
            ui.uis.viewLeft = uicontrol('Style', 'pushbutton',...
                'String','left',...
                'Parent',ui.uis.iCubViewCtrlUIPanel,'Units','normalized','Position', [w 0 w 0.5],...
                'FontSize', 12,'Callback',@ui.view);
            ui.uis.viewSideRight = uicontrol('Style', 'pushbutton',...
                'String','side-right',...
                'Parent',ui.uis.iCubViewCtrlUIPanel,'Units','normalized','Position', [2*w 0.5 w 0.5],...
                'FontSize', 12,'Callback',@ui.view);
            ui.uis.viewSideLeft = uicontrol('Style', 'pushbutton',...
                'String','side-left',...
                'Parent',ui.uis.iCubViewCtrlUIPanel,'Units','normalized','Position', [2*w 0 w 0.5],...
                'FontSize', 12,'Callback',@ui.view);
            ui.uis.viewTopLeft = uicontrol('Style', 'pushbutton',...
                'String','top-left',...
                'Parent',ui.uis.iCubViewCtrlUIPanel,'Units','normalized','Position', [3*w 0 w 0.5],...
                'FontSize', 12,'Callback',@ui.view);
            ui.uis.viewTopRight = uicontrol('Style', 'pushbutton',...
                'String','top-right',...
                'Parent',ui.uis.iCubViewCtrlUIPanel,'Units','normalized','Position', [3*w 0.5 w 0.5],...
                'FontSize', 12,'Callback',@ui.view);
            ui.uis.viewTopFrontal = uicontrol('Style', 'pushbutton',...
                'String','top-front',...
                'Parent',ui.uis.iCubViewCtrlUIPanel,'Units','normalized','Position', [4*w 0.5 w 0.5],...
                'FontSize', 12,'Callback',@ui.view);
            
            %%%%%%%%% Capture Control %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            ui.uis.iCubCaptureUIPanel = uipanel('Title', 'iCubSim Capture Image',...
                'Parent',ui.parentup,...
                'Units','normalized',...
                'Position', [0 0 1 0.2],...
                'FontSize', 12, 'BackgroundColor', 'white');
            ui.uis.captureUI = uicontrol('Style', 'pushbutton',...
                'String','capture',...
                'Parent',ui.uis.iCubCaptureUIPanel,'Units','normalized','Position', [0 0 0.25 1],...
                'FontSize', 12,'Callback',@ui.capture);
            ui.uis.captureName = uicontrol('Style', 'edit',...
                'String','000','Min',1,'Max',1,...
                'Parent',ui.uis.iCubCaptureUIPanel,'Units','normalized','Position', [0.25 0 0.75 1],...
                'FontSize', 12);
        end
        
    end%methods
    
end

