classdef YarpCntPanel < handle
    %YARPCNTPANEL Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        name = 'yarpcnt';
        portname = '/default';
        remoteportname = '/icubSim/command:rpc'
        cntrole = 'out';
        port = [];
        %editable = 0;
        
        connected = 0;
        
        f = [];
        parentup = [];
        position = [0 0 1 1];
        uis = [];
    end
    
    methods
        function ui = YarpCntPanel(parentup, position)
            if nargin < 1
                ui.f = figure('Visible','off','Position',[400,300,800,700]);
                set(ui.f,'MenuBar','none');
                set(ui.f,'Units','normalized');
                set(ui.f,'Name','YarpCntPanel');
                movegui(ui.f,'center');
                set(ui.f,'Visible','on');
                ui.parentup = ui.f;
            else
                ui.parentup = parentup;
            end
            
            if nargin > 1
                ui.position = position;
            end

            ui.initUI();
        end
        
        function delete(ui)
            ui.disconnect();
        end
        
        function b = isconnected(ui)
            b = ui.connected;
        end
        
        function connect(ui,src,evn)
            if ui.connected
                return;
            end
            
            getYarped();
            
            [ui.port ui.connected] = yarpOpenPort(ui.portname);
            if ui.connected
                if strcmp(ui.cntrole, 'out') || strcmp(ui.cntrole, 'output') || strcmp(ui.cntrole, 'o')
                    yarpConnect(char(ui.port.getName), ui.remoteportname);
                    ui.connected = ui.port.getOutputCount();
                else
                    yarpConnect(ui.remoteportname, char(ui.port.getName));
                    ui.connected = ui.port.getInputCount();
                end
            end
            
            if ui.connected
                set(ui.uis.connect, 'Enable', 'off');
                set(ui.uis.disconnect, 'Enable', 'on');
                %set(ui.uis.switchroles, 'Enable', 'off');
            else
                ui.port.close();
                ui.port = [];
            end
        end
        
        function disconnect(ui,src,evn)
            if ~ui.connected
                return;
            end
            if ~isempty(ui.port)
                ui.port.close();
                ui.port = [];
            end
            ui.connected = 0;
            disp(['YarpCntPanel ' ui.name ' disconnected']);
            
            set(ui.uis.connect, 'Enable', 'on');
            set(ui.uis.disconnect, 'Enable', 'off');
            if 0 && ui.editable
                set(ui.uis.inpname, 'Enable', 'on');
                set(ui.uis.outname, 'Enable', 'on');
                set(ui.uis.switchroles, 'Enable', 'on');
            end
        end
        
        function switchroles(ui,src,evn)
            if ~ui.editable
                return;
            end
            
            tmp = get(ui.uis.inpname, 'String');
        end
        
        
        function initUI(ui)
            ui.uis.panel = uipanel('Title', ui.name,...
                'Units','normalized',...
                'Position', ui.position,...
                'Parent',ui.parentup,...
                'FontSize', 12, 'BackgroundColor', 'white');
            if 0
            ui.uis.inpname = uicontrol('Style', 'edit',...
                'String',ui.portname,'Parent',ui.uis.panel,'Units','normalized','Position', [0.8 0.2 0.8 0.2],...
                'FontSize', 12);
            ui.uis.outname = uicontrol('Style', 'edit',...
                'String',ui.remoteportname,'Parent',ui.uis.panel,'Units','normalized','Position', [0.6 0.2 0.8 0.2],...
                'FontSize', 12);
            ui.uis.switchroles = uicontrol('Style', 'pushbutton',...
                'String','<->','Parent',ui.uis.panel,'Units','normalized','Position', [0 0.6 0.2 0.8],...
                'FontSize', 12,'Callback',@ui.switchroles);
            end
            ui.uis.roleinfoTxt = uicontrol('Style', 'pushbutton',...
                'String', 'role:',...
                'Parent',ui.uis.panel,'Units','normalized','Position', [0 0.8 0.2 0.2],'FontSize', 12);
            ui.uis.portinfo1Txt = uicontrol('Style', 'pushbutton',...
                'String', 'this portname:',...
                'Parent',ui.uis.panel,'Units','normalized','Position', [0 0.7 0.2 0.1],'FontSize', 12);
            ui.uis.portinfo2Txt = uicontrol('Style', 'pushbutton',...
                'String', 'remote portname:',...
                'Parent',ui.uis.panel,'Units','normalized','Position', [0 0.6 0.2 0.1],'FontSize', 12);
            ui.uis.roleinfoVal = uicontrol('Style', 'pushbutton',...
                'String', ui.cntrole,...
                'Parent',ui.uis.panel,'Units','normalized','Position', [0.2 0.8 0.8 0.2],'FontSize', 12);
            ui.uis.portinfo1Val = uicontrol('Style', 'pushbutton',...
                'String', ui.portname,...
                'Parent',ui.uis.panel,'Units','normalized','Position', [0.2 0.7 0.8 0.1],'FontSize', 12);
            ui.uis.portinfo2Val = uicontrol('Style', 'pushbutton',...
                'String', ui.remoteportname,...
                'Parent',ui.uis.panel,'Units','normalized','Position', [0.2 0.6 0.8 0.1],'FontSize', 12);
            
            ui.uis.connect = uicontrol('Style', 'pushbutton',...
                'String','connect','Parent',ui.uis.panel,'Units','normalized','Position', [0 0.3 1 0.3],...
                'FontSize', 12,'Callback',@ui.connect);
            ui.uis.disconnect = uicontrol('Style', 'pushbutton',...
                'String','disconnect','Parent',ui.uis.panel,'Units','normalized','Position', [0 0 1 0.3],...
                'FontSize', 12,'Callback',@ui.disconnect, 'Enable', 'off');
        end
        
        function showUI(ga)
            set(ga.uis.panel, 'Title', ga.name);
            set(ga.uis.roleinfoVal, 'String', ga.cntrole);
            set(ga.uis.portinfo1Val, 'String', ga.portname);
            set(ga.uis.portinfo2Val, 'String', ga.remoteportname);
            
            fields = fieldnames(ga.uis);
            for i = 1:numel(fields)
                set(ga.uis.(fields{i}),'visible','on');
            end
        end
        
        function hideUI(ga)
            fields = fieldnames(ga.uis);
            for i = 1:numel(fields)
                set(ga.uis.(fields{i}),'visible','off');
            end
        end
    end
    
end

