classdef rpsRobotSim < rpsIO
    
    properties
        gui = [];
        icubUI = [];
        stateUI = [];
        cntUI = [];
                
        sequenceStepDetection = 5; %3;
        detectionThreshold = 0.6;
        costsNothingDetected = 0.5;
        detectedBuffer = [];
        
        headDownPos= [ -0.2263    0.0002         0    0.0002   -0.0000         0];%Rad
        headHome = [ 0.0023    0.0002         0    0.0002   -0.0000         0];
        rightarmrest = [-0.17453  0.5236 0  0.7854 0 0 0 0.2618 0.5236 0.05236 0 0 0.010472 0.010472 0.005236  0];
        
        executedRPS = 0;
        detectedRPS = 0;
        
        respondedRewardPunishment=0;
        
        negativeSentences = {'','','','','Oh no!','Bluddy bastered','dammit','That suxx','','','',''}
        positiveSentences = {'','','','','Thank you very much','I am awesome','yeei ha','Thats nice','','','',''}
        
        status = 0;
        PRECONNECT = -1;
        WAITFORSTART = 0;
        CONNECTING = 1;
        RUNNING = 2;
        
        cntparams = {
            'MarkerTracker', '/CCDemo/MarkerTracker:i', '/MarkerTracker/marker:o', 'input';...
            'ESpeakServer', '/CCDemo/espeak', '/ESpeakServer/command', 'output';
            'RopsciDetector', '/CCDemo/detector', '/Ropsci/rps:o', 'input';
            'RopsciSequencer_status', '/CCDemo/sequencerstatus', '/RopsciSequencer/status:o', 'input';
            'RopsciSequencer', '/CCDemo/sequencer', '/RopsciSequencer/command', 'output';
            %'MVITE_head', '/CCDemo/MVITE_head:o', '/MVITE/head/state:i', 'output';...
            };
    end
    
    methods
        function app = rpsRobotSim()
            app = app@rpsIO();
        end
        
        function init(app)
            set(app.ser.runbutton, 'Enable', 'off');
            set(app.ser.runbutton, 'Visible', 'off');
            
            app.gui = CDemoGUI(app);
            app.icubUI = ICubUISimRob(app, app.gui.icubPanel);
            
            app.cntUI = YarpCntAssistant(app.cntparams, app.gui.varPanel);
            app.cntUI.hideUI();
            
            app.stateUI = RPSStateMachineUI(app, app.gui.statePanel);
            app.stateUI.setState(app.PRECONNECT);
            
            app.ser.playerReady = 0;
        end
        
        
        function readStimuli(app)           
			%% read ropsci sequencer status
            [bottle str] = yarpRead(app.cntUI.cnts.RopsciSequencer_status.port);
            [bottle str] = yarpRead(app.cntUI.cnts.RopsciSequencer_status.port);
            stat = str2num(str);

            %% Assign modulation if touched and play sound accordingly
			if stat(1) >= 3 && stat(1) <= 5 % && stat(2) >= app.sequenceStepDetection %not currently playing
                
		        app.rewarded = app.isrewarded();
		        app.punished = app.ispunished();
                
                if app.rewarded == 1
                    app.rewarded =0;
                    app.robotRewardOC();
                    app.currentModulation = app.par.positiveReward;
                    if ~app.respondedRewardPunishment
                        yarpWrite(app.cntUI.cnts.ESpeakServer.port, ['say "' app.positiveSentences{randi(length(app.positiveSentences))} '"']);
                        app.respondedRewardPunishment =1;
                    end
                end  
                
                if app.punished == 1
                    app.punished = 0;
                    app.robotPunishmentOC();
                    app.currentModulation = app.par.negativeReward;
                    if ~app.respondedRewardPunishment
                        yarpWrite(app.cntUI.cnts.ESpeakServer.port, ['say "' app.negativeSentences{randi(length(app.negativeSentences))} '"']);
                        app.respondedRewardPunishment =1;
                    end
                end
            end
            %else
			%% read ropsci detector
            if stat(1) >= 0 && stat(1) < 3 %currently playing
                if stat(2) == app.sequenceStepDetection %time to detect something / recognition phase
                    [bottle str] = yarpRead(app.cntUI.cnts.RopsciDetector.port);
                    if strcmp(bottle.get(0).toString_c(), 'none')
                        app.detectedBuffer = [app.detectedBuffer; 0 0 0 app.costsNothingDetected];
                    else
                        tmp = [str2double(bottle.get(1).toString_c()) str2double(bottle.get(2).toString_c()) str2double(bottle.get(3).toString_c())];
                        [maxval maxidx] = max(tmp);
                        tmp = zeros(1,3);
                        tmp(maxidx) = 1;
                        app.detectedBuffer = [app.detectedBuffer; tmp 0];
                    end
                    tmp = mean(app.detectedBuffer,1);
                    [maxval app.detectedRPS] = max(tmp);
                    if maxval > app.detectionThreshold && size(app.detectedBuffer,1) > 4
                        yarpWrite(app.cntUI.cnts.ESpeakServer.port, ['say "you did ' app.labels{app.detectedRPS} ', i did ' app.labels{app.executedRPS} '"']);
                        disp(['detected ' app.labels{app.detectedRPS}]);
                        disp(['executed ' app.labels{app.executedRPS}]);
                        %yarpWrite(app.cntUI.cnts.RopsciSequencer.port, ['set state ' num2str(app.sequenceStepDetection+1)]);
                        %yarpWrite(app.cntUI.cnts.RopsciSequencer.port, 'set state -1');
                        yarpWrite(app.cntUI.cnts.RopsciSequencer.port, 'set state 4'); %pause(0.1)
                        %app.icubUI.driveToPosture('right_arm', app.rightarmrest);
                        %create network input from detected and executed action
                        app.OCstimuli = zeros(app.par.nrOCStimuli,1);
                        app.OCstimuli(3*(app.executedRPS-1)+app.detectedRPS) = 1;
                    end
                end
                if stat(2) > app.sequenceStepDetection
                    tmp = mean(app.detectedBuffer,1);
                    [maxval maxidx] = max(tmp);
                    if maxval <= app.detectionThreshold
                        disp(['could not detect anything reliably, best guess is ' app.labels{maxidx} '   buffer is ' num2str(tmp)]);
                        yarpWrite(app.cntUI.cnts.ESpeakServer.port, 'say "sorry, i could not see, very well"');
                        yarpWrite(app.cntUI.cnts.RopsciSequencer.port, 'set state 3');
                        app.icubUI.setEmotion('neu');
                        app.icubUI.driveToPosture('head',app.headHome);
                    end
                end
            end

        end
        
        function newTurn(app)
            %% Detect if a person is present
            [mbottle mstr] = yarpRead(app.cntUI.cnts.MarkerTracker.port);
            if strcmp(mstr,'empty')
                app.ser.playerReady =0;
            else
                app.ser.playerReady = 1;
            end
            
            %% check sequencer status
            [bottle str] = yarpRead(app.cntUI.cnts.RopsciSequencer_status.port);
            stat = str2num(str);
            
            %% start playing
            if app.ser.playerReady == 1 && app.ser.timeFromLastAction > 8/app.par.samplingTime && stat(1) < 0 %&& ~(stat(1) == 3)  %not yet playing
				yarpWrite(app.cntUI.cnts.ESpeakServer.port, 'say "lets play"');
                app.respondedRewardPunishment =0;
                app.OCstimuli = zeros(app.par.nrOCStimuli,1);
                app.executedRPS = randi(3,1,1); %rember action triggered (or read from sequencer status
                yarpWrite(app.cntUI.cnts.RopsciSequencer.port, ['set state ' num2str(app.executedRPS-1)]);
                app.detectedBuffer = [];
                app.icubUI.setEmotion('neu');
                app.icubUI.driveToPosture('head',app.headDownPos);
                disp('start game...');
            end
        end
        
        function triggerRobotActions(app)
            switch app.ser.whoWins
                case 1
                    yarpWrite(app.cntUI.cnts.ESpeakServer.port, 'say "i win"');
                case 2
                    yarpWrite(app.cntUI.cnts.ESpeakServer.port, 'say "i loose"');
                case 3
                    yarpWrite(app.cntUI.cnts.ESpeakServer.port, 'say "no one wins"');
            end
            pause(0.1)
            app.icubUI.driveToPosture('head',app.headHome);
        end
        
        function robotRewardOC(app)
            app.icubUI.setEmotion('hap');
            app.icubUI.driveToPosture('head',app.headHome);
        end
        
        function robotPunishmentOC(app)
            app.icubUI.setEmotion('sad');
            app.icubUI.driveToPosture('head',app.headHome);
        end
        
        function realTimeMargin(app)
            app.realTimeMargin@rpsIO;
            % pause the execution for the time left
            if app.log.idleTime(app.ser.currentStep) > 0
                pause(app.log.idleTime(app.ser.currentStep))
            end
        end
        
        function stopRunning(app)
            yarpWrite(app.cntUI.cnts.RopsciSequencer.port, 'set state -1');
            app.icubUI.setEmotion('clr');
            app.respondedRewardPunishment =0;
            %TODO stop MVITE models if used
        end
        
        function connect(app,src,evn)
            if get(src, 'Value')
                app.stateUI.setState(app.CONNECTING);
            else
                app.stateUI.setState(app.WAITFORSTART);
            end
        end
        
        function delete(app)
            delete(app.icubUI);
            delete(app.cntUI);
        end
    end
    
end

