/**
 * @file blobTrackerThread.h
 * @brief this file contains the definition of an additional thread that does the job for the main module.
 */

#ifndef _MARKERTRACKER_THREAD_H_
#define _MARKERTRACKER_THREAD_H_

#include "opencv/highgui.h"
#include "opencv/cv.h"


#include <ICLQuick/Common.h>
#include <ICLMarkers/FiducialDetector.h>

//#include <ICLGeom/Geom.h>
//#include <ICLUtils/FPSLimiter.h>

#include <yarp/sig/all.h>
#include <yarp/os/all.h>
#include <iostream>
#include <stdlib.h>
#include <stdio.h>


/**
 * An instance of the class blobTrackerThread processes frames from the camera and estimate the position of the target.
 */

class MarkerTrackerThread : public yarp::os::Thread {
private:
	yarp::sig::ImageOf<yarp::sig::PixelRgb> *frame;//Current frame from the camera

	IplImage* cvFrame;// Current frame from the camera, but converted for OpenCV
	FiducialDetector *fid;

	yarp::os::BufferedPort<yarp::sig::ImageOf<yarp::sig::PixelRgb> > imagePortIn;
	yarp::os::BufferedPort<yarp::os::Bottle> portOut;

	// static application data

public:
	/**
	 * constructor.
	 *
	 */
	MarkerTrackerThread();

	/**
	 * destructor.
	 */
	~MarkerTrackerThread() {}

	bool threadInit();
	void threadRelease();
	void run(); // run the thread. It is the main function
	void onStop();

	std::string inputPortName, outputPortName;

};


#endif  //_MARKERTRACKER_THREAD_H_

//----- end-of-file --- ( next line intentionally left blank ) ------------------
