
/**
 * @file MarkerTrackerThread.cpp
 * @brief implementation of the MarkerTrackerThread (for the blobTrackerModule) methods.
 */

#include <iCub/MarkerTrackerThread.h>
#include <string.h>

using namespace yarp::os;
using namespace yarp::sig;
using namespace std;


MarkerTrackerThread::MarkerTrackerThread() {

}

bool MarkerTrackerThread::threadInit() {
	fid = new FiducialDetector("bch","[0-10]","size=20x20");
	//icl::FiducialDetector fid("bch","[0-10]","size=20x20");
	fid->setConfigurableID("fid");

	//fid->loadMarkers("[0,10]",ParamList("size",Size(96,96)));
	fid->loadMarkers("[0-10]",ParamList("size",Size(84,84)));
	try{
		//	fid->setPropertyValue("border width",2);
		//	fid->setPropertyValue("general.max tilt",10);
		//	fid->setPropertyValue("general.match factor",2);
		//	fid->setPropertyValue("general.optimize edges",1);
		//	fid->setPropertyValue("css.angle-threshold",180);
		//	fid->setPropertyValue("thresh.algorithm","region mean");
		//	fid->setPropertyValue("thresh.mask size",100);
		//	fid->setPropertyValue("quads.minimum region size",400);
		//	fid->setPropertyValue("quads.maximum region size",100000);
		//	fid->setPropertyValue("thresh.algorithm","tiled linear");
		//	fid->setPropertyValue("pose.algorithm","SamplingFine");
		//	fid->setPropertyValue("pp.filter","median");
		//	fid->setPropertyValue("pp.mask size","5x5");
			fid->loadProperties("./conf/configFile.xml");
	}catch(ICLException &e){
		WARNING_LOG("exception caught while setting initial parameters: " << e.what());
	}
	/* opening ports */
	if (!imagePortIn.open(inputPortName.c_str()) ) {
		std::cout << ": unable to open input port " << std::endl;
		return false;  // unable to open; let RFModule know so that it won't rrun
	}

	if (!portOut.open(outputPortName.c_str())) {
		cout  << ": unable to open output port " <<  endl;
		return false;  // unable to open; let RFModule know so that it won't run
	}
	//fid->saveProperties("/homes/alemme/SVN/Projects/icubconditioning/Demo/workspace/MarkerTracker/conf/configFile2.xml");
	return true;
}

void MarkerTrackerThread::run() {
	int time;

	cvNamedWindow("MarkerTrackerViz", CV_WINDOW_NORMAL);
	cvResizeWindow("MarkerTrackerViz", 640, 480);
	cvMoveWindow("MarkerTrackerViz", 0, 0);
	CvFont font;
	//cvInitFont(&font, CV_FONT_HERSHEY_SIMPLEX, 1.0, 1.0, 0, 5, CV_AA);
	cvInitFont(&font, CV_FONT_HERSHEY_SCRIPT_SIMPLEX, 1.0, 1.0, 0, 3, CV_AA); 
	CvScalar txtcolor = cvScalar(0, 0, 0, 0);

	while (isStopping() != true) { // the thread continues to run until isStopping() returns true

		frame = imagePortIn.read(true);

		if(frame!=0) {
			/* Convert a yarpImage into an IplImage (required for using OpenCV functions ) */
			cvFrame = cvCreateImage(cvSize(frame->width(), frame->height()), IPL_DEPTH_8U, 1);
			cvCvtColor((IplImage*)frame->getIplImage(), cvFrame, CV_RGB2GRAY);

			/* Convert a yarpImage into an ICLImage (required for using ICL functions ) */
			Img8u iclimage(Size(frame->width(), frame->height()), formatGray);

			memcpy(iclimage.getDataPtr(0), cvFrame->imageData,(cvFrame->imageSize)*sizeof(unsigned char) );

			const std::vector<Fiducial> &fids = fid->detect(&iclimage);


			/*** The main loop. It processes a frame per period ***/
			//Bottle output;
			Bottle& output = portOut.prepare();
			output.clear();
			if (fids.size()) {
				output.addInt(fids.size());
				for(unsigned int i=0;i<fids.size();++i){
					Point32f c = fids[i].getCenter2D();

					output.addInt(atoi(fids[i].getName().c_str()));
					output.addDouble(c.x - frame->width()/2);
					output.addDouble(c.y - frame->height()/2);

					CvPoint p = cvPoint((int) c.x, (int) c.y);
					cvDrawCircle(cvFrame, p, 15, CV_RGB(0, 0, 255), -1);
					cvPutText(cvFrame, fids[i].getName().c_str(), cvPoint((int) c.x - 10, (int) c.y + 10), &font, txtcolor);
				}
				//std::cout << fids.size() << std::endl;
				cout << output.toString().c_str() << endl;
			} else {
				output.addString("empty");
			}
			//cout << "writing " << output.toString().c_str() << endl;
			portOut.write();


			/* Draw a circle at the center of the frame */
			cvDrawCircle(cvFrame, cvPoint(( frame->width() )/2, ( frame->height() )/2), 5, CV_RGB(0, 255, 0), 1);
			cvShowImage("MarkerTrackerViz", cvFrame);
			cvWaitKey(10);
			cvReleaseImage(&cvFrame);

			//time = clock() - time;
			//std::cout<< ((float)time)/CLOCKS_PER_SEC << " s per iteration" << endl;
		}
	}  //while
}


void MarkerTrackerThread::threadRelease() {
	/* for example, delete dynamically created data-structures */
}

void MarkerTrackerThread::onStop() {
	imagePortIn.interrupt();
	portOut.interrupt();
	imagePortIn.close();
	portOut.close();
}


