#include "opencv/highgui.h"
#include "opencv/cv.h"


#include <ICLQuick/Common.h>
#include <ICLMarkers/FiducialDetector.h>

//#include <ICLGeom/Geom.h>
//#include <ICLUtils/FPSLimiter.h>

#include <yarp/sig/all.h>
#include <yarp/os/all.h>
#include <iostream>

#include <yarp/sig/all.h>
#include <yarp/os/all.h>
#include <yarp/os/RFModule.h>
#include <yarp/os/Network.h>
#include <yarp/os/Thread.h>
using namespace yarp::os;
using namespace yarp::sig;
using namespace std;
// static application data
GUI gui("hsplit");

// the global detector class ("bch" markers)
// here, using marker ids {100,101, .. 200}
icl::FiducialDetector fid("bch","[0-10]","size=20x20");
yarp::os::BufferedPort<yarp::sig::ImageOf<yarp::sig::PixelRgb> > imagePortIn;
yarp::os::BufferedPort<yarp::os::Bottle> portOut;
std::string moduleName;
std::string robotName;
std::string robotPortName;
std::string inputPortName;
std::string outputPortName;
std::string handlerPortName;
std::string cameraConfigFilename;
float  fxLeft,  fyLeft;          // focal length
float  fxRight, fyRight;         // focal length
float  cxLeft,  cyLeft;          // coordinates of the principal point
float  cxRight, cyRight;         // coordinates of the principal point
int thresholdValue;
ResourceFinder rf;
yarp::os::Port handlerPort;      //a port to handle messages

void init(){
	Network yarp;
	fid.loadMarkers("[0-10]",ParamList("size",Size(84,84)));
	// set the configurable ID (see icl::Configurable class documentation)
	fid.setConfigurableID("fid");

	// create the GUI
	gui << "draw[@handle=draw]"           // create drawing component
			<< "prop(fid)[@maxsize=18x100]"   // create the property widged for 'fid'
			<< "!show";                       // show the main widget

	// initialize the grabber from given program argument

	/* Process all parameters from both command-line and .ini file */
	/* get the module name which will form the stem of all module port names */
	moduleName            = rf.check("name",
			Value("MarkerTrackerModule"),
			"module name (string)").asString();


	/* now, get the rest of the parameters */
	/*
	 * get the robot name which will form the stem of the robot ports names
	 * and append the specific part and device required
	 */
	robotName             = rf.check("robot",
			Value("icub"),
			"Robot name (string)").asString();

	robotPortName         = "/" + robotName + "/head";



	/* get the name of the input and output ports, automatically prefixing the module name by using getName() */
	inputPortName         = "";
	inputPortName        += "/image:i";

	outputPortName        = "";
	outputPortName       += "/image:o";

	/* get the threshold value */
	thresholdValue        = rf.check("threshold",
			Value(8),
			"Key value (int)").asInt();

	/* opening ports */
	if (!imagePortIn.open(inputPortName.c_str()) ) {
		std::cout << ": unable to open input port " << std::endl;

	}

	if (!portOut.open(outputPortName.c_str())) {
		cout  << ": unable to open output port " <<  endl;

	}

}


// working loop (automatically looped in the working thread)
void run(){

	IplImage* cvFrame;// Current frame from the camera, but converted for OpenCV
	yarp::sig::ImageOf<yarp::sig::PixelRgb> *frame;//Current frame from the camera
	while(true){
		frame = imagePortIn.read(true);

		if(frame!=0) {

			//time = clock();


			/* Convert a yarpImage into an IplImage (required for using OpenCV functions ) */
			//cvFrame = cvCreateImage(cvSize(frame->width(), frame->height()), IPL_DEPTH_8U, 3 );
			//cvCvtColor((IplImage*)frame->getIplImage(), cvFrame, CV_RGB2BGR);


			/* Convert a yarpImage into an IplImage (required for using OpenCV functions ) */
			cvFrame = cvCreateImage(cvSize(frame->width(), frame->height()), IPL_DEPTH_8U, 1);
			cvCvtColor((IplImage*)frame->getIplImage(), cvFrame, CV_RGB2GRAY);
			cout  << "image size received " << frame->width() << "x" <<frame->height() <<  endl;
			/* Convert a yarpImage into an ICLImage (required for using ICL functions ) */
			Img8u iclimage(Size(frame->width(), frame->height()), formatGray);

			memcpy(iclimage.getDataPtr(0), cvFrame->imageData,(cvFrame->imageSize)*sizeof(unsigned char) );;

			const std::vector<Fiducial> &fids = fid.detect(&iclimage);

			// get a handle to the "draw" component
			static DrawHandle draw = gui["draw"];

			// visualize the image
			draw = iclimage;

			// draw marker detection results
			draw->linewidth(2);
			for(unsigned int i=0;i<fids.size();++i){
				Point32f c = fids[i].getCenter2D();
				float rot = fids[i].getRotation2D();

				draw->color(255,0,0,255);
				draw->linestrip(fids[i].getCorners2D());
				draw->text(fids[i].getName(),c.x,c.y,10);
				draw->color(0,255,0,255);
				draw->fill(0,255,0,255);
				draw->arrow(c, c+Point32f( cos(rot), sin(rot))*25, 5);
			}

			/*** The main loop. It processes a frame per period ***/
		/*	//Bottle output;
			Bottle& output = portOut.prepare();
			output.clear();
			if (fids.size()) {
				output.addInt(fids.size());
				for(unsigned int i=0;i<fids.size();++i){
					Point32f c = fids[i].getCenter2D();

					output.addInt(atoi(fids[i].getName().c_str()));
					output.addDouble(c.x - frame->width()/2);
					output.addDouble(c.y - frame->height()/2);

					cvDrawCircle(cvFrame, cvPoint((int) c.x, (int) c.y), 10, CV_RGB(0, 0, 255), -1);
				}
				//std::cout << fids.size() << std::endl;
				cout << output.toString().c_str() << endl;
			} else {
				output.addString("empty");
			}
			//cout << "writing " << output.toString().c_str() << endl;
			portOut.write();*/
			// update the visualization
			draw.render();

			/* Draw a circle at the center of the frame */
			cvDrawCircle(cvFrame, cvPoint(( frame->width() )/2, ( frame->height() )/2), 5, CV_RGB(0, 255, 0), 1);
			//cvShowImage("tracker", cvFrame);
			//cvWaitKey(10);
			cvReleaseImage(&cvFrame);

			//time = clock() - time;
			//std::cout<< ((float)time)/CLOCKS_PER_SEC << " s per iteration" << endl;

		}
	}  //while
}

// main method
int main(int n, char **ppc){
	/* prepare and configure the resource finder */

	rf.setVerbose(true);
	rf.setDefaultConfigFile("MarkerTracker.ini"); //overridden by --from parameter
	rf.setDefaultContext("./conf");   //overridden by --context parameter
	rf.configure("ICUB_ROOT", n, ppc);
	return ICLApp(n,ppc,"",init,run).exec();
}
