/**
 * @file MarkerTrackerModule.cpp
 * @brief implementation of the MarkerTrackerModule methods following the RFModule standards.
 */

/*
 * Audit Trail
 * -----------
 *  .... if any
 */ 

#include "iCub/Vite.h"
#include <yarp/math/Math.h>
#include <yarp/math/Rand.h>
#include <iostream>

using namespace yarp::sig;
using namespace yarp::math;
using namespace std;

Vite::Vite( Vector& xd, Vector& xs, double alpha, double beta, double noise){
	this->xd =xd;
	this->xs =xs;

	this->alpha =alpha;
	this->beta =beta;
	this->noise = noise;
}
Vite::~Vite(){

}

void Vite::setNewTask( Vector& xd, Vector& xs){
	this->xd =xd;
	this->xs =xs;
	cout<<"VITE set new Task"<<endl;
	cout<<this->toString()<<endl;
}
Vector Vite::step( Vector& x,  Vector& v){

	Vector d = xd - xs;
	//cout<< "[VITE] xs "<< xs.toString(2,1)<< " xd " <<xd.toString(2,1);
	Vector randomVec =Rand::vector(xd.length());

	Vector u = xd - x + noise * norm(xd - x) * randomVec;
	//	cout<< " u " << u.toString(2,1) <<endl<< " x " << x.toString(2,1)<<endl << "random Vector " << randomVec.toString(2,1)<<endl;
	double g = yarp::math::norm(d - u);
	//cout<< " g " << g;
	Vector a = alpha * (-1 * v + beta * g * u);
	//cout<< " a " << a.toString(5,1);
	v +=  a;
	return v;
	//cout<< " v " << v.toString(2,1)<< endl;
	//cout<< " alpha " << alpha<< " beta " << beta<< " noise "<< noise<<endl;
}
void  Vite::setAlpha(double value){
	this->beta = value;
}
void  Vite::setBeta(double value){
	this->beta = value;
}

string Vite::toString(){
	string xdName = string(xd.toString(4,1));
	string xsName = string(xs.toString(4,1));
	string returnName= "[VITE]";
	return returnName + "Target: " + xdName + "\n starting Position " + xsName;// + "\n alpha " + alpha + " beta " + beta ;
}
