close all;
clear all;

%% load and prepare data
load('../data/rpsn.mat');

labelCol = 1;
dataCols = 2:size(D,2);

classes = {'rock', 'paper', 'scissor', 'noise'};

X = D(:,dataCols);
L = D(:,labelCol);
Y = label2ook(L);
Y(Y==0) = -1;
numSamples = size(X,1);


%% learner params
learnerSpec = {'hidDim', 200; 'reg', 1e-3; 'BIP', 1; 'mu', 0.2};


%% create and train network
net = ELM(size(X,2), size(Y,2), learnerSpec);
net.init(X);
net.train(X, Y);

%test
[Yhat H] = net.apply(X);
Lhat = ook2label(Yhat);

%eval
CR = sum(L==Lhat)/numSamples
fprintf('done!\n');

figure; hist(H(:));


%% save network
write2file('../ELM/ELM_info.vec', [net.inpDim net.outDim net.hidDim]);
write2file('../ELM/ELM_a.vec', net.a);
write2file('../ELM/ELM_b.vec', net.b);
write2file('../ELM/ELM_Winp.mat', net.Winp);
write2file('../ELM/ELM_aWinp.mat', repmat(net.a', 1, net.inpDim) .* net.Winp);
write2file('../ELM/ELM_Wout.mat', net.Wout');

