
/**
 * @file RopsciThread.cpp
 * @brief implementation of the RopsciThread (for the Ropsci) methods.
 */

#include <iCub/RopsciThread.h>
#include <iCub/RopsciModule.h>
#include <string.h>

using namespace yarp::os;
using namespace yarp::sig;
using namespace std;
using namespace cv;


RopsciThread::RopsciThread(RopsciModule* module) {
	this->module = module;
}

bool RopsciThread::threadInit() {

    return true;
}

void RopsciThread::run() {
  int time;
  
  /*** The main loop. It processes a frame per period ***/
  while (isStopping() != true) { // the thread continues to run until isStopping() returns true
    //std::cout << "[Ropsci::run]" << std::endl;

	//get input image
	this->module->frame = this->module->imagePortIn.read(true);
	if (this->module->frame == 0) {
		sleep(1);
		continue;
	}

	this->module->img = (IplImage*) this->module->frame->getIplImage();
	cvCvtColor(this->module->img, this->module->img, CV_RGB2BGR); // correct color order
	//TODO buggy
	/*if ((this->module->img->width != 320) && (this->module->img->height != 240)) {
		IplImage *tmp = cvCreateImage( cvSize(320, 240), this->module->img->depth, this->module->img->nChannels);
		cvResize(this->module->img, tmp);
		//cvCopy(tmp, this->module->img);
		//cvReleaseImage(&tmp);
		//cvReleaseImage(&this->module->img);
		this->module->img = tmp;
	}*/

	//EXAMPLE with image from fil
	//this->module->img = imread("data/rock.jpg", CV_LOAD_IMAGE_COLOR); //openvc2
	//this->module->img = cvLoadImage("data/scissor.jpg");
	
//	cvShowImage("Ropsci", this->module->img);
	//cvWaitKey(20);

	//prepare output bottle
	Bottle& output = this->module->portOut.prepare();
	output.clear();

	//keep image in grayscale for boxing
	this->module->convertToGreyScale(this->module->img);
	this->module->skinColorFilter();
	bool skindetected = this->module->boundingBox();
	
	//check whether relevant region is detected (TODO check size of boundingbox)
	if (skindetected) {
		//std::cout << "[Ropsci::skindetected]" << std::endl;
		cvShowImage("Ropsci", this->module->roiImg);
		cvWaitKey(20);

		this->module->gaborFilter();
		int winner = this->module->classify();
		std::string wstr;
		switch (winner) {
			case Ropsci_Rock:		wstr = "rock";		break;
			case Ropsci_Paper:		wstr = "paper";		break;
			case Ropsci_Scissor:	wstr = "scissor";	break;
			default: 				wstr = "none";		break;
		}
		output.addString(wstr.c_str());
		output.addDouble(this->module->lout[0]);
		output.addDouble(this->module->lout[1]);
		output.addDouble(this->module->lout[2]);
		cvReleaseImage(&this->module->gaborImg);
	} else {
		output.addString("none");
	}
	cvReleaseImage(&this->module->colorMaskImg);

	//publish output
	//cout << "writing " << output.toString().c_str() << endl;
	this->module->portOut.write();

	sleep(0.1);
	//cvReleaseImage(&this->module->img); //segfault
  }  //while
}


void RopsciThread::threadRelease() {
   /* for example, delete dynamically created data-structures */
}

void RopsciThread::onStop() {

}


