
/**
 * @file RopsciSequencerThread.cpp
 * @brief implementation of the RopsciSequencerThread (for the RopsciSequencer) methods.
 */

#include <iCub/RopsciSequencerThread.h>
#include <iCub/RopsciSequencerModule.h>
#include <string.h>

using namespace yarp::os;
using namespace yarp::sig;
using namespace std;


RopsciSequencerThread::RopsciSequencerThread(RopsciSequencerModule* module) {
	this->module = module;
}

bool RopsciSequencerThread::threadInit() {

    return true;
}

void RopsciSequencerThread::run() {
  double time;
  int currentstate = -1;

  /*** The main loop. It processes a frame per period ***/
  while (isStopping() != true) { // the thread continues to run until isStopping() returns true
       
	//get state
	currentstate = this->module->state;

	//inform world of current status
	Bottle& output = this->module->statusOut.prepare();
	output.clear();
	output.addInt(currentstate);
	output.addInt(this->module->step);
	this->module->statusOut.write();

	//handle state (idle, sequence index)
	if (currentstate < 0) {//IDLE
		//std::cout << "[" << this->module->getName() << "] " << "idle" << std::endl;
		this->module->step = 0;
		this->module->executed = false;
		sleep(0.1);//in seconds
		this->module->lastTimeSwitched = Time::now();//in seconds
		continue;
	}

	//check elapsed time and increment
	time = Time::now() - this->module->lastTimeSwitched;
	if (time >= this->module->pausesecs[currentstate][this->module->step]) {
		if ((time - this->module->pausesecs[currentstate][this->module->step]) > 0.5) {
			std::cout << "[" << this->module->getName() << "] " << "delay in processing! desired delay: " << this->module->pausesecs[currentstate][this->module->step] << "s  actual delay: " << time << "s" << std::endl;
		}
		this->module->step++;
		this->module->executed = false;
		this->module->lastTimeSwitched = Time::now();
	} else {
		if (this->module->executed) {
			sleep(0.01);
			continue;
		}
	}

	if (this->module->step >= (int) this->module->jointangleseqs[currentstate].size()) {
		if (this->module->loop) {
			//repeat from beginning
			this->module->step = 0;
		} else {
			//go idle
			std::cout << "[" << this->module->getName() << "] " << "sequence " << this->module->seqnames[currentstate] << " finished, going idle" << std::endl;
			this->module->state = -1;
			currentstate = -1;
			continue;
		}
	}

	std::cout << "[" << this->module->getName() << "] " << "executing sequence " << this->module->seqnames[currentstate] << ", step " << this->module->step+1 << "/" << this->module->jointangleseqs[currentstate].size() << std::endl;

	//send joint angles
	if (this->module->jointangleseqs[currentstate][this->module->step].length() > 0) {
		Bottle& output = this->module->jasOut.prepare();
		output.clear();
		output.addString("set");
		output.addString("target");
		yarp::sig::Vector v = this->module->jointangleseqs[currentstate][this->module->step];
		for (int i=0; i<v.length(); i++)	output.addDouble(v[i]);
		std::cout << output.toString() << std::endl;
		this->module->jasOut.write();
	}

	//send espeak call
	if (this->module->voiceseqs[currentstate][this->module->step] != "none") {
		std::string cmd = "\"" + this->module->voiceseqs[currentstate][this->module->step] + "\"";
		Bottle& output = this->module->esOut.prepare();
		output.clear();
		output.addString("say");
		output.addString(cmd.c_str());
		this->module->esOut.write();
	}

	this->module->executed = true;

  }  //while
}


void RopsciSequencerThread::threadRelease() {
   /* for example, delete dynamically created data-structures */
}

void RopsciSequencerThread::onStop() {

}


