/**
 * @file SimpleGazeCtrlModule.h
 * @brief this file contains the definition of the main module code (that is the module yarp implmentation using the RFModule class.
 */

#ifndef _SimpleGazeCtrl_MODULE_H_
#define _SimpleGazeCtrl_MODULE_H_

#include <iostream>
#include <string>

#include <yarp/sig/all.h>
#include <yarp/math/Math.h>
#include <yarp/os/all.h>
//#include <yarp/os/RFModule.h>
//#include <yarp/os/Network.h>
//#include <yarp/os/Thread.h>
#include <yarp/dev/ControlBoardInterfaces.h>
#include <yarp/dev/PolyDriver.h>
 
// within project includes  
#include <iCub/SimpleGazeCtrlThread.h>

#define RAD2DEG    (180.0/M_PI)
#define DEG2RAD    (M_PI/180.0)

/**
 * here you need to document your classes.
 */
class SimpleGazeCtrlModule : public yarp::os::RFModule {
	friend class SimpleGazeCtrlThread;

public:
	static const int IDLE = 0;
	static const int RUN = 1;
	static const int MARKER = 0;
	static const int PXL = 1;

private:
    std::string moduleName;
    std::string robotName; 
    std::string bodypart;  

	std::string handlerPortName;
    std::string inpMarkerPortName;
    std::string outStatusPortName;

    yarp::os::Port handlerPort;
	yarp::os::BufferedPort<yarp::os::Bottle> inpCommand;
	yarp::os::BufferedPort<yarp::os::Bottle> inpMarker;
	yarp::os::BufferedPort<yarp::os::Bottle> outStatus;

	yarp::dev::PolyDriver polydriver;
	yarp::dev::IEncoders        *iencs;
    yarp::dev::IVelocityControl *ivel;

	int state;
	int status;
	int inpsrc;

	int markerToTrack;
	float k_pan;
	float k_tilt;
	float k_rollcomp;
	float d_home;
	double d_lowpass;

	int numStepsNoInp;
	int maxNumStepsNoInp;

	yarp::sig::Vector qhead;
	yarp::sig::Vector dqhead;
	yarp::sig::Vector dqheadOld;

    /* pointer to a new thread to be created and started in configure() and stopped in close() */
    SimpleGazeCtrlThread *dThread;

public:
    bool configure(yarp::os::ResourceFinder &rf); // configure all the module parameters and return true if successful
    bool interruptModule();                       // interrupt, e.g., the ports 
	bool openPorts();
	bool openRobotInterface();
    bool close();                                 // close and shut down the module
    bool respond(const yarp::os::Bottle& command, yarp::os::Bottle& reply);
    double getPeriod(); 
    bool updateModule();
	bool receiveCommands(bool blocking=false);

	double rollCompensation(double pan, double tilt);
};


#endif // __SimpleGazeCtrl_MODULE_H__

//----- end-of-file --- ( next line intentionally left blank ) ------------------

