#!/bin/bash
if [ $# -ne 1 ] ; then
  echo -e "usage:\n\tcreateModule modulename"
  echo -e "\t(e.g.) ./createModule MyModule"
  exit 1
fi

if [ -d $1 ] ; then
    echo "error: modulename $1 already exists"
    exit 1
fi

if [ -e $1 ] ; then
    echo "error: file with name $1 already exists"
    exit 1
fi

PROJECT=$1

# Create folder root dir
echo -n "Creating module ${PROJECT} ...                      "
mkdir -p ${PROJECT}
echo "done"

# Create folder lib dir
echo -n "Creating sub folders doc lib src obj application... "
for T in src include build conf 'include/iCub' ; do
    mkdir -p ${PROJECT}/$T ;
done
echo "done"

echo -n "Copying CMakeLists.txt ...                         "
cp templates/CMakeLists.txt ${PROJECT}/CMakeLists.txt
echo "done"

echo -n "Creating initial src files ...                         "
cp templates/main.cpp ${PROJECT}/src/
cp templates/Module.h ${PROJECT}/include/iCub/${PROJECT}Module.h
cp templates/Thread.h ${PROJECT}/include/iCub/${PROJECT}Thread.h
cp templates/Module.cpp ${PROJECT}/src/${PROJECT}Module.cpp
cp templates/Thread.cpp ${PROJECT}/src/${PROJECT}Thread.cpp
echo "done"

echo -n "Creating initial conf file ...                               "
cp  templates/stdconf.ini ${PROJECT}/conf/${PROJECT}.ini
echo "done"

echo "Adapt modulename in files"
find -P $PROJECT/ -name "CMakeLists.txt" -exec sed -i "s|iCubTemplateModule|$PROJECT|g" {} \; -exec echo "replacing in {}" \;
find -P $PROJECT/include/iCub/ -name "*.h" -exec sed -i "s|iCubTemplateModule|$PROJECT|g" {} \; -exec echo "replacing in {}" \;
find -P $PROJECT/src/ -name "*.cpp" -exec sed -i "s|iCubTemplateModule|$PROJECT|g" {} \; -exec echo "replacing in {}" \;
find -P $PROJECT/conf/ -name "*.ini" -exec sed -i "s|iCubTemplateModule|$PROJECT|g" {} \; -exec echo "replacing in {}" \;

