classdef RPS < handle
    %RPS Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        gui = [];
        icubUI = [];
        stateUI = [];
        cntUI = [];
                
        playerReady = 0;
        sequenceStepDetection = 5; %3;
        detectionThreshold = 0.6;
        costsNothingDetected = 0.5;
        detectedBuffer = [];
        labels = {'rock', 'paper', 'scissor', 'none'};
        
        headDownPos= [ -0.2263    0.0002         0    0.0002   -0.0000         0];%Rad
        headHome = [ 0.0023    0.0002         0    0.0002   -0.0000         0];
        rightarmrest = [-0.17453  0.5236 0  0.7854 0 0 0 0.2618 0.5236 0.05236 0 0 0.010472 0.010472 0.005236  0];
        
        executedRPS = 0;
        detectedRPS = 0;
        
        status = 0;
        PRECONNECT = -1;
        WAITFORSTART = 0;
        CONNECTING = 1;
        RUNNING = 2;
        
        cntparams = {
            'MarkerTracker', '/CCDemo/MarkerTracker:i', '/MarkerTracker/marker:o', 'input';...
            'ESpeakServer', '/CCDemo/espeak', '/ESpeakServer/command', 'output';
            'RopsciDetector', '/CCDemo/detector', '/Ropsci/rps:o', 'input';
            'RopsciSequencer_status', '/CCDemo/sequencerstatus', '/RopsciSequencer/status:o', 'input';
            'RopsciSequencer', '/CCDemo/sequencer', '/RopsciSequencer/command', 'output';
            };
    end
    
    methods
        function app = RPS()          
            app.gui = CDemoGUI(app);
            app.icubUI = ICubUISimRob(app, app.gui.icubPanel);
            
            app.cntUI = YarpCntAssistant(app.cntparams, app.gui.varPanel);
            app.cntUI.hideUI();
            
            app.stateUI = RPSStateMachineUI(app, app.gui.statePanel);
            app.stateUI.setState(app.PRECONNECT);
        end
        

        function running(app,src,evn)
            if get(src, 'Value')
                while get(src, 'Value')
                    app.newTurn();
                    app.readStimuli();
                    pause(0.01);
                end
                yarpWrite(app.cntUI.cnts.RopsciSequencer.port, 'set state -1');
                app.icubUI.setEmotion('clr');
            else
                %run button released
                yarpWrite(app.cntUI.cnts.RopsciSequencer.port, 'set state -1');
                app.icubUI.setEmotion('clr');
            end
        end
        
        function readStimuli(app)           
			%% read ropsci sequencer status
            [bottle str] = yarpRead(app.cntUI.cnts.RopsciSequencer_status.port);
            stat = str2num(str);

			%% read ropsci detector
            if stat(1) >= 0 && stat(1) < 3 %currently playing
                if stat(2) == app.sequenceStepDetection %time to detect something / recognition phase
                    [bottle str] = yarpRead(app.cntUI.cnts.RopsciDetector.port);
                    if strcmp(bottle.get(0).toString_c(), 'none')
                        app.detectedBuffer = [app.detectedBuffer; 0 0 0 app.costsNothingDetected];
                    else
                        tmp = [str2num(bottle.get(1).toString_c()) str2num(bottle.get(2).toString_c()) str2num(bottle.get(3).toString_c())];
                        [maxval maxidx] = max(tmp);
                        tmp = zeros(1,3);
                        tmp(maxidx) = 1;
                        app.detectedBuffer = [app.detectedBuffer; tmp 0];
                    end
                    tmp = mean(app.detectedBuffer,1);
                    [maxval app.detectedRPS] = max(tmp);
                    if maxval > app.detectionThreshold && size(app.detectedBuffer,1) > 4
                        yarpWrite(app.cntUI.cnts.ESpeakServer.port, ['say "you did ' app.labels{app.detectedRPS} ' i did ' app.labels{app.executedRPS} '"']);
                        disp(['detected ' app.labels{app.detectedRPS}]);
                        disp(['executed ' app.labels{app.executedRPS}]);
                        yarpWrite(app.cntUI.cnts.RopsciSequencer.port, 'set state 3'); pause(0.1);
                        %app.icubUI.driveToPosture('right_arm', app.rightarmrest);
                        app.respond();
                    end
                end
                if stat(2) > app.sequenceStepDetection && ~(stat(1) == 3)
                    tmp = mean(app.detectedBuffer,1);
                    [maxval maxidx] = max(tmp);
                    if maxval <= app.detectionThreshold
                        disp(['could not detect anything reliably, best guess is ' app.labels{maxidx} '   buffer is ' num2str(tmp)]);
                        yarpWrite(app.cntUI.cnts.ESpeakServer.port, 'say "could not detect anything"');
                        yarpWrite(app.cntUI.cnts.RopsciSequencer.port, 'set state 3');
                    end
                end
            end

        end
        
        function newTurn(app)
            %% Detect if a person is present
            [mbottle mstr] = yarpRead(app.cntUI.cnts.MarkerTracker.port);
            if strcmp(mstr,'empty')
                app.playerReady =0;
            else
                app.playerReady = 1;
            end
            
            %% check sequencer status
            [bottle str] = yarpRead(app.cntUI.cnts.RopsciSequencer_status.port);
            stat = str2num(str);
            
            %% start playing
            if app.playerReady == 1 && ...
                    ...%app.ser.timeFromLastAction > 7/app.par.samplingTime && 
                    stat(1) < 0 % ~(stat(1) == 3))  %not yet playing
                disp('start game...');
				yarpWrite(app.cntUI.cnts.ESpeakServer.port, 'say "lets play"');
                app.executedRPS = randi(3,1,1); %rember action triggered (or read from sequencer status
                yarpWrite(app.cntUI.cnts.RopsciSequencer.port, ['set state ' num2str(app.executedRPS-1)]);
                app.detectedBuffer = [];
                app.icubUI.setEmotion('neu');
                app.icubUI.driveToPosture('head',app.headDownPos);
            end
        end
        
        function respond(app)
            %% determine who wins
            whoWins = 0;
            if app.detectedRPS == app.executedRPS
                whoWins = 3;
            elseif app.detectedRPS == 1 && app.executedRPS == 2
                whoWins = 1;
            elseif app.detectedRPS == 1 && app.executedRPS == 3
                whoWins = 2;
            elseif app.detectedRPS == 2 && app.executedRPS == 1
                whoWins = 2;
            elseif app.detectedRPS == 2 && app.executedRPS == 3
                whoWins = 1;
            elseif app.detectedRPS == 3 && app.executedRPS == 1
                whoWins = 1;
            elseif app.detectedRPS == 3 && app.executedRPS == 2
                whoWins = 2;
            end
                
            %% trigger actions
            switch whoWins
                case 1
                    yarpWrite(app.cntUI.cnts.ESpeakServer.port, 'say "i win"');
                    app.icubUI.setEmotion('hap');
                case 2
                    yarpWrite(app.cntUI.cnts.ESpeakServer.port, 'say "you win"');
                    app.icubUI.setEmotion('sad');
                case 3
                    yarpWrite(app.cntUI.cnts.ESpeakServer.port, 'say "noone wins"');
                    app.icubUI.setEmotion('neu');
            end
            app.icubUI.driveToPosture('head',app.headHome);
        end
        
        function connect(app,src,evn)
            if get(src, 'Value')
                app.stateUI.setState(app.CONNECTING);
            else
                app.stateUI.setState(app.WAITFORSTART);
            end
        end
        
        function delete(app)
            delete(app.icubUI);
            delete(app.cntUI);
        end
    end
    
end

